@@ -2,4 +2,4 @@
 # Copyright 2015 Roger Marsh
 # Licence: See LICENCE (BSD licence)
 
-"""Database engine independent importers for ECF reference data."""
+"""ChessTab database modules common to all database engine interfaces."""
