@@ -7,8 +7,12 @@
 Game* classes replace PGN* classes in ChessTab version 4.3.
 
 """
-from pgn_read.core.game import Game, GameIndicateCheck
-from pgn_read.core.squares import Squares
+import re
+
+from pgn_read.core.game import Game, suffix_annotations
+from pgn_read.core.game_indicate_check import GameIndicateCheck
+from pgn_read.core.movetext_parser import MoveText
+from pgn_read.core.squares import fen_squares
 from pgn_read.core.constants import (
     FEN_WHITE_KING,
     FEN_WHITE_QUEEN,
@@ -23,15 +27,32 @@ from pgn_read.core.constants import (
     FEN_BLACK_KNIGHT,
     FEN_BLACK_PAWN,
     FEN_WHITE_ACTIVE,
+    TAG_RESULT,
+    DEFAULT_TAG_RESULT_VALUE,
+    OTHER_SIDE,
+    FEN_BLACK_ACTIVE,
+    PGN_DOT,
+    SUFFIX_ANNOTATION_TO_NAG,
+    PGN_MAXIMUM_LINE_LENGTH,
+    PGN_LINE_SEPARATOR,
+    PGN_TOKEN_SEPARATOR,
 )
 
-from chessql.core.constants import ANY_WHITE_PIECE_NAME, ANY_BLACK_PIECE_NAME
-
 from .constants import (
     TAG_OPENING,
     REPERTOIRE_TAG_ORDER,
     REPERTOIRE_GAME_TAGS,
     MOVE_NUMBER_KEYS,
+    START_COMMENT,
+    ERROR_START_COMMENT,
+    ESCAPE_END_COMMENT,
+    HIDE_END_COMMENT,
+    END_COMMENT,
+    ANY_WHITE_PIECE_NAME,
+    ANY_BLACK_PIECE_NAME,
+    EMPTY_SQUARE_NAME,
+    ALWAYS_MATCH,
+    WHITE_PIECE_NAMES,
 )
 
 MAP_PGN_PIECE_TO_CQL_COMPOSITE_PIECE = {
@@ -48,6 +69,11 @@ MAP_PGN_PIECE_TO_CQL_COMPOSITE_PIECE = {
     FEN_BLACK_KNIGHT: ANY_BLACK_PIECE_NAME,
     FEN_BLACK_PAWN: ANY_BLACK_PIECE_NAME,
 }
+_ALWAYS_MATCH = frozenset(ALWAYS_MATCH)
+
+# Remove multiple leading '[%.*]' and adjacent \s* in '{.*}'.
+structured_comment_re = re.compile(r"^{(?:\s*\[\s*%.*\])+\s*")
+NULL_PGN_COMMENT = "{}"
 
 
 class GameDisplayMoves(GameIndicateCheck):
@@ -74,6 +100,153 @@ class GameDisplayMoves(GameIndicateCheck):
         super().modify_board_state(position_delta)
         self.moves.append((self._text[-1], self._ravstack[-1][-1]))
 
+    def get_export_pgn_rav_no_structured_comments(self):
+        """Return Export format PGN version of game with RAVs but no comments.
+
+        This method will be removed without notice in future.  It seems more
+        convenient and clearer to use the called methods directly.
+
+        Where check or checkmate moves are present the text is not in export
+        format unless generated by the GameIndicateCheck class, because these
+        indicators are not included in the text otherwise.
+
+        """
+        return (
+            self.get_seven_tag_roster_tags(),
+            self.get_export_movetext_without_structured_comments(),
+            self.get_non_seven_tag_roster_tags(),
+        )
+
+    def get_export_movetext_without_structured_comments(self):
+        """Return movetext without structured comments in pgn export format.
+
+        Where check or checkmate moves are present the text is not in export
+        format unless generated by the GameIndicateCheck class, because these
+        indicators are not included in the text otherwise.
+
+        """
+        fullmove_number, active_color = self._set_movetext_indicators()
+        movetext = ["\n"]
+        if self._movetext_offset is None:
+            return movetext
+        length = 0
+        insert_fullmove_number = True
+        fnas = [[fullmove_number, active_color]]
+        _attm = self._add_token_to_movetext
+        termination = self._tags.get(TAG_RESULT, DEFAULT_TAG_RESULT_VALUE)
+        for mvt in self._text[self._movetext_offset :]:
+            if mvt.startswith("{"):
+                if mvt != NULL_PGN_COMMENT:
+                    mvt = structured_comment_re.sub("{", mvt)
+                    if mvt == NULL_PGN_COMMENT:
+                        continue
+                for word in mvt.split():
+                    length = _attm(word, movetext, length)
+                insert_fullmove_number = True
+            elif mvt.startswith("$"):
+                length = _attm(mvt, movetext, length)
+            elif mvt.startswith(";"):
+                if len(mvt) + length >= PGN_MAXIMUM_LINE_LENGTH:
+                    movetext.append(PGN_LINE_SEPARATOR)
+                else:
+                    movetext.append(PGN_TOKEN_SEPARATOR)
+                movetext.append(mvt)
+                length = 0
+                insert_fullmove_number = True
+            elif mvt == "(":
+                length = _attm(mvt, movetext, length)
+                fnas[-1] = [fullmove_number, active_color]
+                active_color = OTHER_SIDE[active_color]
+                if active_color == FEN_BLACK_ACTIVE:
+                    fullmove_number -= 1
+                fnas.append([fullmove_number, active_color])
+                insert_fullmove_number = True
+            elif mvt == ")":
+                length = _attm(mvt, movetext, length)
+                del fnas[-1]
+                fullmove_number, active_color = fnas[-1]
+                insert_fullmove_number = True
+            elif mvt == termination:
+                length = _attm(mvt, movetext, length)
+            elif active_color == FEN_WHITE_ACTIVE:
+                length = _attm(
+                    str(fullmove_number) + PGN_DOT, movetext, length
+                )
+                srchm = suffix_annotations.search(mvt)
+                if srchm:
+                    mvt = mvt[: srchm.start()]
+                length = _attm(mvt, movetext, length)
+                if srchm:
+                    length = _attm(
+                        SUFFIX_ANNOTATION_TO_NAG[srchm.group()],
+                        movetext,
+                        length,
+                    )
+                active_color = OTHER_SIDE[active_color]
+                insert_fullmove_number = False
+            else:
+                if insert_fullmove_number:
+                    length = _attm(
+                        str(fullmove_number) + PGN_DOT * 3, movetext, length
+                    )
+                    insert_fullmove_number = False
+                srchm = suffix_annotations.search(mvt)
+                if srchm:
+                    mvt = mvt[: srchm.start()]
+                length = _attm(mvt, movetext, length)
+                if srchm:
+                    length = _attm(
+                        SUFFIX_ANNOTATION_TO_NAG[srchm.group()],
+                        movetext,
+                        length,
+                    )
+                active_color = OTHER_SIDE[active_color]
+                fullmove_number += 1
+        return "".join(movetext)
+
+
+class _Game(Game):
+    """Override the PGN error notification and recovery methods.
+
+    Errors detected in PGN movetext are hidden by wrapping all tokens to end
+    of variation, which may be rest of game if error is in main line, in a
+    comment which starts and ends with a presumed unlikely character sequence.
+    The '}' in any '{}' comments which happen to get wrapped a changed to a
+    distinct presumed unlikely character sequence so the wrapped '}' tokens do
+    not terminate the wrapping comment.
+
+    """
+
+    def __init__(self):
+        """Delegate then set 'errors_hidden_in_comments' to None."""
+        super().__init__()
+        self.errors_hidden_in_comments = None
+
+    def pgn_error_notification(self):
+        """Insert error '{' before movetext token which causes PGN error."""
+        if self._movetext_offset is not None:
+            self._text.append(START_COMMENT + ERROR_START_COMMENT)
+            self.errors_hidden_in_comments = True
+
+    def pgn_error_recovery(self):
+        """Insert error '}' before token which ends the scope of a PGN error.
+
+        This token will be a ')' or one of the game termination markers.
+
+        """
+        if self._movetext_offset is not None:
+            self._text.append(ESCAPE_END_COMMENT + END_COMMENT)
+
+    def pgn_mark_comment_in_error(self, comment):
+        """Return comment with '}' replaced by a presumed unlikely sequence.
+
+        One possibility is to wrap the error in a '{...}' comment.  The '}'
+        token in any wrapped commment would end the comment wrapping the error
+        prematurely, so replace with HIDE_END_COMMENT.
+
+        """
+        return comment.replace(END_COMMENT, HIDE_END_COMMENT)
+
 
 class GameMove(Game):
     """Generate data structures to verify moves returned from Chess Engines.
@@ -161,7 +334,7 @@ def get_position_string(
 
     """
     del halfmove_clock, fullmove_number
-    squares = Squares.squares
+    squares = fen_squares
     for square, piece in board.items():
         piece.set_square(square)
     return (
@@ -336,9 +509,9 @@ class GameTags(Game):
             self._movetext_offset = len(self._text)
         self._text.append(match.group())
         try:
-            self.repeat_board_state(self._position_deltas[-1])
+            self.repeat_board_state()
         except IndexError:
-            self.add_board_state_none(None)
+            self.add_board_state_none()
 
     def append_token(self, match):
         """Ignore valid non-tag token which does not change board state.
@@ -348,9 +521,9 @@ class GameTags(Game):
 
         """
         try:
-            self.repeat_board_state(self._position_deltas[-1])
+            self.repeat_board_state()
         except IndexError:
-            self.add_board_state_none(None)
+            self.add_board_state_none()
 
     append_reserved = append_token
 
@@ -362,9 +535,9 @@ class GameTags(Game):
 
         """
         if self._movetext_offset is None:
-            self.append_token_and_set_error(match)
+            self._append_token_and_set_error(match)
             return
-        self.reset_board_state(None)
+        self.set_board_state(None)
         self._ravstack.append(None)
         self._text.append(match.group())
         self._state_stack.append(self._state)
@@ -376,23 +549,11 @@ class GameTags(Game):
         place in game score.
 
         """
-        if self._state is not None or self._movetext_offset is None:
-            self.append_token_and_set_error(match)
-            return None
-
-        if self._movetext_offset is None:
-            self.append_token_and_set_error(match)
-            return None
-        if len(self._ravstack) == 1:
-            self.append_token_and_set_error(match)
-            return None
-        del self._ravstack[-1]
-        del self._state_stack[-1]
-        self._state = self._state_stack[-1]
-
+        if not self._reset_after_end_rav(match):
+            return
         self.set_board_state(None)
         self._text.append(match.group())
-        return True
+        return
 
     def append_other_or_disambiguation_pgn(self, match):
         """Ignore token.
@@ -439,14 +600,13 @@ class GameTags(Game):
                 del self._state_stack[-1]
                 self._state = self._state_stack[-1]
             else:
-
                 # Cannot call append_end_rav() method because it tests some
                 # conditions that should be true when errors are absent.
                 if self._movetext_offset is None:
-                    self.append_token_and_set_error(match)
+                    self._append_token_and_set_error(match)
                     return
                 if len(self._ravstack) == 1:
-                    self.append_token_and_set_error(match)
+                    self._append_token_and_set_error(match)
                     return
                 del self._ravstack[-1]
                 del self._state_stack[-1]
@@ -479,28 +639,29 @@ class GameRepertoireTags(GameTags):
         return True
 
 
-class GameUpdate(Game):
+class GameUpdate(_Game):
     """Prepare indicies after each token has been processed."""
 
-    # self.positions, and the three similar, renamed to self.positionkeys.
+    # self.positions renamed to self.positionkeys.
     # self.movenumber is changed to self.halfmovenumber.
     def __init__(self):
-        """Delegate then prepare to collect positions and piece locations."""
+        """Delegate then prepare to collect positions."""
         super().__init__()
         self.positionkeys = []
-        self.piecesquaremovekeys = []
-        self.piecemovekeys = []
-        self.squaremovekeys = []
+        self.piecesquarekeys = set()
         self.halfmovenumber = None
-        self.variationnumber = None
-        self.currentvariation = None
-        self._variation = None
 
     # Replaces self.set_position_fen(self, fen=None)
     def set_initial_board_state(self, position_delta):
-        """Initialize PGN score parser with Forsyth Edwards Notation position.
+        """Delegate then initialize board state and index entries.
 
-        fen defaults to the starting position for a game of chess.
+        The piece location field of the FEN (Forsyth Edwards Notation) is
+        replaced by <Piece instance, square name> tuples for each piece
+        in position_delta
+
+        self.piecesquarekeys is initialised with values for all 64 squares,
+        meaning each square had the indicated state at some point during
+        the game.
 
         """
         super().set_initial_board_state(position_delta)
@@ -508,85 +669,231 @@ class GameUpdate(Game):
             self.halfmovenumber = [(self._fullmove_number - 1) * 2]
         else:
             self.halfmovenumber = [self._fullmove_number * 2 - 1]
-        self.variationnumber = [0]
-        self._variation = "".join(
-            _convert_integer_to_length_hex(i) for i in self.variationnumber
-        )
-
-    def reset_board_state(self, position_delta):
-        """Delegate then append variation number to fit ravstack level."""
-        super().reset_board_state(position_delta)
-        if len(self._ravstack) > len(self.variationnumber):
-            self.variationnumber.append(0)
-
-    # Why '[len(self._ravstack)-1]' rather than '[-1]'?  The '[len()]' version
-    # came from PGNUpdate in chesstab, without the -1 adjustment, and seems to
-    # work.
-    def set_board_state(self, position_delta):
-        """Delegate then increment variation number.
-
-        Both numeric and string variation numbers are kept in step in case
-        there is another variation at this level.  For example:
-        '... Ba7 ) ( Nf4 ...'.
-        """
-        super().set_board_state(position_delta)
-        self.variationnumber[len(self._ravstack) - 1] += 1
-        self._variation = "".join(
-            _convert_integer_to_length_hex(i) for i in self.variationnumber
-        )
+        piecesquarekeys = self.piecesquarekeys
+        piecesquarekeys.update(_ALWAYS_MATCH)
+        for square in set(fen_squares).difference(
+            {s[1] for s in position_delta[0]}
+        ):
+            piecesquarekeys.add(EMPTY_SQUARE_NAME + square)
+        for piece, square in position_delta[0]:
+            piecesquarekeys.update(
+                (
+                    str(piece),
+                    (
+                        ANY_WHITE_PIECE_NAME
+                        if piece.name in WHITE_PIECE_NAMES
+                        else ANY_BLACK_PIECE_NAME
+                    )
+                    + square,
+                    piece.name,
+                )
+            )
 
     def modify_board_state(self, position_delta):
-        """Delegate then modify board state and add index entries."""
+        """Delegate then modify board state and add index entries.
+
+        position_delta contains before, and after, move FEN descriptions
+        of the board with <square name, Piece instance> tuples for the
+        pieces removed and added to the board by the move replacing the
+        FEN piece location field.
+
+        self.piecesquarekeys is extended to show these squares had the
+        indicated state at some point in the game.
+
+        """
         super().modify_board_state(position_delta)
         if len(self._ravstack) != len(self.halfmovenumber):
             while len(self._ravstack) < len(self.halfmovenumber):
                 self.halfmovenumber.pop()
-                self.variationnumber.pop()
             while len(self._ravstack) > len(self.halfmovenumber):
                 self.halfmovenumber.append(self.halfmovenumber[-1])
-                self.variationnumber.append(0)
-            self._variation = "".join(
-                _convert_integer_to_length_hex(i) for i in self.variationnumber
-            )
         self.halfmovenumber[-1] += 1
-        movenumber = _convert_integer_to_length_hex(self.halfmovenumber[-1])
-        piecesquaremovekeys = self.piecesquaremovekeys
-        piecemovekeys = self.piecemovekeys
-        squaremovekeys = self.squaremovekeys
         pieces = [""] * 64
-        bits = []
-        mnv = movenumber + self._variation
+        bits = 0
         for piece in self._piece_placement_data.values():
-            piece_name = piece.name
-            piece_square = piece.square
-            square_name = piece_square.name
-            pieces[piece_square.number] = piece_name
-            bits.append(piece_square.bit)
-
-            # piecesquaremovekeys.append(mnv + piece_name + square_name)
-            # squaremovekeys.append(mnv + mp[piece_name] + square_name)
-
-            # If 'square piece' is better order than 'piece square'
-            piecesquaremovekeys.append(mnv + square_name + piece_name)
-            squaremovekeys.append(
-                (
-                    mnv
-                    + square_name
-                    + MAP_PGN_PIECE_TO_CQL_COMPOSITE_PIECE[piece_name]
-                )
-            )
-
+            pieces[piece.square.number] = piece.name
+            bits += piece.square.bit
         pieces = "".join(pieces)
-        for piece_name in set(pieces):
-            piecemovekeys.append(mnv + piece_name)
         delta_after = position_delta[1]
         self.positionkeys.append(
-            sum(bits).to_bytes(8, "big").decode("iso-8859-1")
+            bits.to_bytes(8, "big").decode("iso-8859-1")
             + pieces
             + delta_after[1]
             + delta_after[3]
             + delta_after[2]
         )
+        delta_before = position_delta[0]
+        piecesquarekeys = self.piecesquarekeys
+        if len(delta_before[0]) == len(delta_after[0]):
+            for square, _ in delta_before[0]:
+                piecesquarekeys.add(EMPTY_SQUARE_NAME + square)
+        else:
+            for square, _ in delta_before[0]:
+                for square_after, _ in delta_after[0]:
+                    if square == square_after:
+                        break
+                else:
+                    piecesquarekeys.add(EMPTY_SQUARE_NAME + square)
+        for square, piece in delta_after[0]:
+            piecesquarekeys.update(
+                (
+                    str(piece),
+                    (
+                        ANY_WHITE_PIECE_NAME
+                        if piece.name in WHITE_PIECE_NAMES
+                        else ANY_BLACK_PIECE_NAME
+                    )
+                    + square,
+                    piece.name,  # Promote to piece not in initial position.
+                )
+            )
+
+
+class GameMergeUpdate(GameUpdate):
+    """Prepare indicies after each token has been processed.
+
+    Assume PGN movetext gives no moves leaving king of moving side in check.
+    """
+
+    def is_side_off_move_in_check(self):
+        """Override and return False."""
+        return False
+
+
+class GameUpdatePosition(_Game):
+    """Prepare position index after each token has been processed.
+
+    Assume PGN movetext gives no moves leaving king of moving side in check.
+    """
+
+    def __init__(self):
+        """Delegate then prepare to collect positions."""
+        super().__init__()
+        self.positionkeys = []
+
+    def modify_board_state(self, position_delta):
+        """Delegate then modify board state and add index entries."""
+        super().modify_board_state(position_delta)
+        pieces = [""] * 64
+        bits = 0
+        for piece in self._piece_placement_data.values():
+            piece_name = piece.name
+            piece_square = piece.square
+            pieces[piece_square.number] = piece_name
+            bits += piece_square.bit
+        delta_after = position_delta[1]
+        self.positionkeys.append(
+            bits.to_bytes(8, "big").decode("iso-8859-1")
+            + "".join(pieces)
+            + delta_after[1]
+            + delta_after[3]
+            + delta_after[2]
+        )
+
+    def is_side_off_move_in_check(self):
+        """Override and return False."""
+        return False
+
+
+class GameUpdatePieceLocation(_Game):
+    """Prepare piece location indicies after each token has been processed.
+
+    Assume PGN movetext gives no moves leaving king of moving side in check.
+    """
+
+    def __init__(self):
+        """Delegate then prepare to collect piece locations."""
+        super().__init__()
+        self.piecesquarekeys = set()
+        self.halfmovenumber = None
+
+    # Replaces self.set_position_fen(self, fen=None)
+    def set_initial_board_state(self, position_delta):
+        """Delegate then initialize index entries.
+
+        The piece location field of the FEN (Forsyth Edwards Notation) is
+        replaced by <Piece instance, square name> tuples for each piece
+        in position_delta
+
+        self.piecesquarekeys is initialised with values for all 64 squares,
+        meaning each square had the indicated state at some point during
+        the game.
+
+        """
+        super().set_initial_board_state(position_delta)
+        if self._active_color == FEN_WHITE_ACTIVE:
+            self.halfmovenumber = [(self._fullmove_number - 1) * 2]
+        else:
+            self.halfmovenumber = [self._fullmove_number * 2 - 1]
+        piecesquarekeys = self.piecesquarekeys
+        piecesquarekeys.update(_ALWAYS_MATCH)
+        for square in set(fen_squares).difference(
+            {s[1] for s in position_delta[0]}
+        ):
+            piecesquarekeys.add(EMPTY_SQUARE_NAME + square)
+        for piece, square in position_delta[0]:
+            piecesquarekeys.update(
+                (
+                    str(piece),
+                    (
+                        ANY_WHITE_PIECE_NAME
+                        if piece.name in WHITE_PIECE_NAMES
+                        else ANY_BLACK_PIECE_NAME
+                    )
+                    + square,
+                    piece.name,
+                )
+            )
+
+    def modify_board_state(self, position_delta):
+        """Delegate then add index entries.
+
+        position_delta contains before, and after, move FEN descriptions
+        of the board with <square name, Piece instance> tuples for the
+        pieces removed and added to the board by the move replacing the
+        FEN piece location field.
+
+        self.piecesquarekeys is extended to show these squares had the
+        indicated state at some point in the game.
+
+        """
+        super().modify_board_state(position_delta)
+        if len(self._ravstack) != len(self.halfmovenumber):
+            while len(self._ravstack) < len(self.halfmovenumber):
+                self.halfmovenumber.pop()
+            while len(self._ravstack) > len(self.halfmovenumber):
+                self.halfmovenumber.append(self.halfmovenumber[-1])
+        self.halfmovenumber[-1] += 1
+        delta_after = position_delta[1]
+        delta_before = position_delta[0]
+        piecesquarekeys = self.piecesquarekeys
+        if len(delta_before[0]) == len(delta_after[0]):
+            for square, _ in delta_before[0]:
+                piecesquarekeys.add(EMPTY_SQUARE_NAME + square)
+        else:
+            for square, _ in delta_before[0]:
+                for square_after, _ in delta_after[0]:
+                    if square == square_after:
+                        break
+                else:
+                    piecesquarekeys.add(EMPTY_SQUARE_NAME + square)
+        for square, piece in delta_after[0]:
+            piecesquarekeys.update(
+                (
+                    str(piece),
+                    (
+                        ANY_WHITE_PIECE_NAME
+                        if piece.name in WHITE_PIECE_NAMES
+                        else ANY_BLACK_PIECE_NAME
+                    )
+                    + square,
+                    piece.name,  # Promote to piece not in initial position.
+                )
+            )
+
+    def is_side_off_move_in_check(self):
+        """Override and return False."""
+        return False
 
 
 class GameUpdateEstimate(GameUpdate):
@@ -625,3 +932,29 @@ def _convert_integer_to_length_hex(integer):
     except IndexError:
         base16 = hex(integer)
         return str(len(base16) - 2) + base16[2:]
+
+
+class GameMoveText(MoveText):
+    """Customize methods of MoveText."""
+
+    def is_pgn_valid(self):
+        """Return True if the tags in the game are valid.
+
+        Override because the tag values matter rather than their existence.
+
+        """
+        return self.is_tag_roster_valid()
+
+
+class GameStore(_Game, GameIndicateCheck):
+    """Add structures to support writing PGN moves to database."""
+
+    def _append_decorated_text(self, movetext):
+        """Append movetext plus appropriate check indicator to self._text."""
+        self._text.append(movetext)
+        self._append_check_indicator()
+
+    def _append_decorated_castles_text(self, movetext):
+        """Append movetext plus appropriate check indicator to self._text."""
+        self._text.append(movetext)
+        self._append_check_indicator()
