@@ -50,7 +50,7 @@ class UCI:
     def __init__(self):
         """Set data stuctures to control multiple UCI chess engines."""
         self.engine_counter = 0
-        self.uci_drivers_index = dict()
+        self.uci_drivers_index = {}
 
         # Avoid "OSError: [WinError 535] Pipe connected"  at Python3.3 running
         # under Wine on FreeBSD 10.1 by disabling the UCI functions.
@@ -69,11 +69,11 @@ class UCI:
         #    self.uci_drivers_reply = None
         self.uci_drivers_reply = multiprocessing.Queue()
 
-        self.uci_drivers = dict()
-        self.uci_drivers_fen = dict()
-        self.uci_active_engines = dict()
+        self.uci_drivers = {}
+        self.uci_drivers_fen = {}
+        self.uci_active_engines = {}
         self.ui_analysis_queue = None
-        self.position_analysis = dict()
+        self.position_analysis = {}
         self._use_ucinewgame = True
         self._clear_hash = True
         self._go_depth = 10
@@ -86,9 +86,9 @@ class UCI:
         self.analysis_data_source = None
 
         # These are new and will replace some of above attributes.
-        self.positions_pending = dict()
-        self.engine_queues = dict()
-        self.clear_hash_after_bestmove = dict()
+        self.positions_pending = {}
+        self.engine_queues = {}
+        self.clear_hash_after_bestmove = {}
         self.set_option_on_empty_queues = set()
 
     def kill_engine(self, number):
@@ -102,7 +102,6 @@ class UCI:
         engine_interface.driver.join()
         engine_name = engine_interface.parser.name
         if engine_name in self.uci_active_engines:
-
             # The command 'stockfish argument' starts stockfish but the engine
             # quits immediately citing 'Unknown command: argument'.
             # These engine_name entries do not exist in this case.
@@ -243,7 +242,6 @@ class UCI:
             and self.is_positions_pending_empty()
         ):
             if ReservedOptionNames.Hash in self.set_option_on_empty_queues:
-
                 # Postpone Hash action until all engines ready.
                 for engine_interface in self.uci_drivers.values():
                     eip = engine_interface.parser
@@ -374,8 +372,7 @@ class UCI:
                 )
                 if depth >= self._go_depth and multipv >= mpv:
                     return False
-                if mpv < multipv:
-                    multipv = mpv
+                multipv = min(multipv, mpv)
 
         # Put commands to do analysis on engine command queue.
         # If analysis already exists do not reduce depth; multipv is already
@@ -565,7 +562,7 @@ class UCI:
 
         eng_a = Analysis()
         eng_a.scale = {
-            engine_name: (min([int(d[2]) for d in lines]), len(lines))
+            engine_name: (min(int(d[2]) for d in lines), len(lines))
         }
         eng_a.position = fen
         eng_a.variations = {engine_name: [(d[1], d[3]) for d in lines]}
@@ -580,7 +577,7 @@ class UCI:
                     erc.value.scale.update(eng_a.scale)
                     erc.value.variations.update(eng_a.variations)
                     inserter = RecordEdit(erc, rec)
-                    inserter.set_data_source(asd, inserter.on_data_change)
+                    inserter.set_data_source(source=asd)
                     inserter.edit()
                     break
             else:
@@ -590,7 +587,7 @@ class UCI:
                     erc.value.scale.update(eng_a.scale)
                     erc.value.variations.update(eng_a.variations)
                     inserter = RecordEdit(erc, rec)
-                    inserter.set_data_source(asd, inserter.on_data_change)
+                    inserter.set_data_source(source=asd)
                     inserter.edit()
                 else:
                     value = self.analysis_record.value
@@ -599,7 +596,7 @@ class UCI:
                     value.position = eng_a.position
                     value.variations = eng_a.variations
                     inserter = RecordEdit(self.analysis_record, None)
-                    inserter.set_data_source(asd, None)
+                    inserter.set_data_source(source=asd)
                     self.analysis_record.set_database(
                         inserter.get_data_source().dbhome
                     )
