@@ -12,10 +12,11 @@ this class because their Text widgets come from solentware_grid.gui.DataRow.
 import tkinter
 import enum
 
-from solentware_misc.gui.exceptionhandler import ExceptionHandler
+from solentware_bind.gui.bindings import Bindings
 
 from .eventspec import EventSpec
 from .displayitems import DisplayItemsStub
+from .eventbinding import BlankTextEventBinding
 
 
 # 'Tag' in these names refers to tags in Tk Text widgets, not PGN tags.
@@ -25,6 +26,11 @@ from .displayitems import DisplayItemsStub
 # other subclasses do not.
 # GameEdit uses all the names.
 class NonTagBind(enum.Enum):
+    """Enumerate the tkinter.Text tag state.
+
+    The tag state is used to control the event bindings for the widget.
+    """
+
     # Current token is a move.
     NO_EDITABLE_TAGS = 1
     NO_CURRENT_TOKEN = 2
@@ -36,8 +42,7 @@ class NonTagBind(enum.Enum):
     SELECT_VARIATION = 6
 
 
-class BlankText(ExceptionHandler):
-
+class BlankText(BlankTextEventBinding, Bindings):
     """Create Text widget with configuration shared by subclasses.
 
     The subclasses are cqltext.CQLText, querytext.QueryText, score.Score,
@@ -90,51 +95,72 @@ class BlankText(ExceptionHandler):
             command=self.score.yview,
         )
         self.score.configure(yscrollcommand=self.scrollbar.set)
+        self.takefocus_widget = None
 
         # Keyboard actions do nothing by default.
-        self.set_keypress_binding(switch=False)
-        self.set_event_bindings_score(self.get_menubar_events())
+        self._set_keypress_binding(switch=False)
+        self.set_event_bindings_score(self._get_menubar_events())
 
         # The popup menus used by all subclasses.
         self.inactive_popup = None
+        self.primary_activity_popup = None
+
+    def _init_takefocus_widget(self, widget):
+        """Initialize inactive_popup to popup if currently None."""
+        # Allow callers to avoid the attribute-defined-outside-init message
+        # if setting directly causes message to be given.
+        self.takefocus_widget = widget
+
+    def _init_inactive_popup(self, popup):
+        """Initialize inactive_popup to popup if currently None."""
+        # Allow callers to avoid the attribute-defined-outside-init message;
+        # access-member-before-definition message is stopped too.
+        # The assert ensures some __init__ has set inactive_popup to None,
+        # although the real purpose is to insist it is not set already.
+        assert hasattr(self, "inactive_popup") and self.inactive_popup is None
+        self.inactive_popup = popup
 
     def set_event_bindings_score(self, bindings=(), switch=True):
         """Set bindings if switch is True or unset the bindings."""
         ste = self.try_event
         for sequence, function in bindings:
-            self.score.bind(
-                sequence[0], ste(function) if switch and function else ""
+            self.bind(
+                self.score,
+                sequence[0],
+                function=ste(function) if switch and function else "",
             )
 
-    def set_keypress_binding(self, function=None, bindings=(), switch=True):
+    def _set_keypress_binding(self, function=None, bindings=(), switch=True):
         """Set bindings to function if switch is True or disable keypress."""
         if switch and function:
             stef = self.try_event(function)
             for sequence in bindings:
-                self.score.bind(sequence[0], stef)
+                self.bind(self.score, sequence[0], function=stef)
         else:
             stekb = self.try_event(self.press_break)
             for sequence in bindings:
-                self.score.bind(sequence[0], stekb)
+                self.bind(self.score, sequence[0], function=stekb)
 
-    def get_menubar_events(self):
+    def _get_menubar_events(self):
         """Return tuple of event binding definitions passed for menubar."""
         return ((EventSpec.score_enable_F10_menubar, self.press_none),)
 
-    def press_break(self, event=None):
+    @staticmethod
+    def press_break(event=None):
         """Do nothing and prevent event handling by next handlers."""
+        del event
         return "break"
 
-    def press_none(self, event=None):
+    @staticmethod
+    def press_none(event=None):
         """Do nothing and allow event to be handled by next handler."""
-        return None
+        del event
 
     # This method arose when seeking clarity in the way popup menus were set,
     # and replaces lots of 'add_command' calls scattered all over.
-    # Long term, either this method or add_cascade_menu_to_popup will do all.
-    def set_popup_bindings(self, popup, bindings=(), index=tkinter.END):
+    # Long term, either this method or _add_cascade_menu_to_popup will do all.
+    def _set_popup_bindings(self, popup, bindings=(), index=tkinter.END):
         """Insert bindings in popup before index in popup."""
-
         # Default index is tkinter.END which seems to mean insert at end of
         # popup, not before last entry in popup as might be expected from the
         # way expressed in the 'Tk menu manual page' for index command.  (The
@@ -150,15 +176,18 @@ class BlankText(ExceptionHandler):
                 accelerator=accelerator[2],
             )
 
-    def give_focus_to_widget(self, event=None):
+    @staticmethod
+    def give_focus_to_widget(event=None):
         """Do nothing and return 'break'.  Override in subclasses as needed."""
+        del event
         return "break"
 
-    def get_F10_popup_events(self, top_left, pointer):
-        """Return tuple of event definitions to post popup menus at top left
-        of focus widget and at pointer location within application widget.
+    @staticmethod
+    def get_f10_popup_events(top_left, pointer):
+        """Return tuple of event definitions to post popup menus.
 
-        top_left and pointer are functions.
+        top_left is a method to post the menu at top left corner of widget.
+        pointer is a method to post the menu at current pointer location.
 
         """
         return (
@@ -167,10 +196,29 @@ class BlankText(ExceptionHandler):
         )
 
     # Subclasses with database interfaces may override method.
-    def create_database_submenu(self, menu):
-        return None
+    @staticmethod
+    def _create_database_submenu(menu):
+        """Do nothing.
 
-    def post_menu(self, menu, create_menu, allowed=True, event=None):
+        Subclasses should override this method as needed.
+
+        """
+        # pylint message assignment-from-none is reported when the return
+        # value is assigned to a name.
+        # The message is not reported if 'return None' is replaced like:
+        # none = None
+        # return none
+        del menu
+
+    def _post_menu(self, menu, create_menu, allowed=True, event=None):
+        """Post the popup menu at current pointer location in widget.
+
+        create_menu creates a popup menu if menu is None.
+        event supplies the screen location for popup menu top left corner.
+        allowed is True, default, permits display of menu.
+
+        """
+        del event
         if menu is None:
             menu = create_menu()
         if not allowed:
@@ -180,9 +228,15 @@ class BlankText(ExceptionHandler):
         # So 'Control-F10' does not fire 'F10' (menubar) binding too.
         return "break"
 
-    def post_menu_at_top_left(
-        self, menu, create_menu, allowed=True, event=None
-    ):
+    @staticmethod
+    def post_menu_at_top_left(menu, create_menu, allowed=True, event=None):
+        """Post the popup menu at top left in widget.
+
+        create_menu creates a popup menu if menu is None.
+        event supplies the screen location for popup menu top left corner.
+        allowed is True, default, permits display of menu.
+
+        """
         if menu is None:
             menu = create_menu()
         if not allowed:
@@ -192,14 +246,14 @@ class BlankText(ExceptionHandler):
         # So 'Shift-F10' does not fire 'F10' (menubar) binding too.
         return "break"
 
-    def is_active_item_mapped(self):
-        """ """
+    def _is_active_item_mapped(self):
+        """Return True if this widget is visible and is the active item."""
         if self.items.is_mapped_panel(self.panel):
             if self is not self.items.active_item:
                 return False
         return True
 
-    def bind_for_primary_activity(self, switch=True):
+    def _bind_for_primary_activity(self, switch=True):
         """Set (switch True) or clear bindings for main actions when active.
 
         If bool(switch) is true, clear the most recently set bindings first.
@@ -208,7 +262,7 @@ class BlankText(ExceptionHandler):
         if switch:
             self.token_bind_method[self._most_recent_bindings](self, False)
             self._most_recent_bindings = NonTagBind.NO_EDITABLE_TAGS
-        self.set_primary_activity_bindings(switch=switch)
+        self._bind_for_set_primary_activity_bindings(switch)
 
     def bind_for_initial_state(self, switch=True):
         """Clear the most recently set bindings if bool(switch) is True.
@@ -225,56 +279,30 @@ class BlankText(ExceptionHandler):
     # Dispatch dictionary for token binding selection.
     # Keys are the possible values of self._most_recent_bindings.
     token_bind_method = {
-        NonTagBind.NO_EDITABLE_TAGS: bind_for_primary_activity,
+        NonTagBind.NO_EDITABLE_TAGS: _bind_for_primary_activity,
         NonTagBind.INITIAL_BINDINGS: bind_for_initial_state,
     }
 
-    def create_primary_activity_popup(self):
+    def _create_primary_activity_popup(self):
+        """Create and return popup menu with primary and database commands.
+
+        The primary commands are set by subclasses and should be related
+        to their main function.
+
+        By default no database commands are set.  Subclasses should
+        override the create_database_submanu method as needed.
+
+        """
+        # pylint message access-member-before-definition.
+        # Initialize primary_activity_popup moved to blanktext from
+        # enginetext module.
         assert self.primary_activity_popup is None
         popup = tkinter.Menu(master=self.score, tearoff=False)
-        self.set_popup_bindings(popup, self.get_primary_activity_events())
-        database_submenu = self.create_database_submenu(popup)
-        if database_submenu:
+        self._set_popup_bindings_get_primary_activity_events(popup)
+        # pylint message assignment-from-none is false positive.
+        # However it is sensible to do an isinstance test.
+        database_submenu = self._create_database_submenu(popup)
+        if isinstance(database_submenu, tkinter.Menu):
             popup.add_cascade(label="Database", menu=database_submenu)
         self.primary_activity_popup = popup
         return popup
-
-    def get_button_events(self, buttonpress1=None, buttonpress3=None):
-        """Return tuple of buttonpress event bindings.
-
-        buttonpress1 and buttonpress3 default to self.press_none().
-
-        """
-        if buttonpress1 is None:
-            buttonpress1 = self.press_none
-        if buttonpress3 is None:
-            buttonpress3 = self.press_none
-        return self.get_modifier_buttonpress_suppression_events() + (
-            (EventSpec.buttonpress_1, buttonpress1),
-            (EventSpec.buttonpress_3, buttonpress3),
-        )
-
-    # Take a snapshot of the tkinter.Text widget bound to self.score.  It is
-    # intended for problem tracing.  It is saved as a sibling of ErrorLog.
-    # At a convenient time dump_text_widget will become a method in the
-    # solentware_misc.gui.exceptionhandler.ExceptionHandler class.
-    def dump_text_widget(self, textwidget, filename=None):
-        if filename is None:
-            filename = "dumptextwidget"
-        import os
-        import datetime
-
-        filename = "_".join(
-            (
-                filename,
-                datetime.datetime.now(datetime.timezone.utc).isoformat(),
-            )
-        )
-        with open(
-            os.path.join(
-                os.path.dirname(self.get_error_file_name()), filename
-            ),
-            "w",
-        ) as f:
-            for t in textwidget.dump("1.0", tkinter.END):
-                f.write(repr(t) + "\n")
