@@ -40,7 +40,7 @@ from pgn_read.core.constants import (
     FEN_WHITE_PIECES,
     FEN_BLACK_PIECES,
 )
-from pgn_read.core.squares import Squares
+from pgn_read.core.squares import fen_squares
 
 from . import constants
 from ..core.constants import NOPIECE
@@ -202,15 +202,15 @@ class Board(Bindings):
         if boardfont:
             self.boardfont = boardfont
 
-        self.squares = dict()
-        self.boardsquares = dict()
+        self.squares = {}
+        self.boardsquares = {}
         # self.boardfont is name of named font or a font instance
         try:
             self.font = tkinter.font.nametofont(self.boardfont).copy()
         except (AttributeError, tkinter.TclError):
             self.font = self.boardfont.copy()
         self.container = tkinter.Frame(
-            master=master, cnf=dict(width=0, height=0)
+            master=master, cnf={"width": 0, "height": 0}
         )
         self.bind(
             self.container,
@@ -219,7 +219,7 @@ class Board(Bindings):
         )
         self.board = tkinter.Frame(
             master=self.container,
-            cnf=dict(borderwidth=boardborder, relief=tkinter.SUNKEN),
+            cnf={"borderwidth": boardborder, "relief": tkinter.SUNKEN},
         )
         board = self.board
         boardsquares = self.boardsquares
@@ -262,8 +262,7 @@ class Board(Bindings):
             piece_chars = _piece_chars[0]
         else:
             piece_chars = _piece_chars[1]
-        for index in self.squares:
-            piece = self.squares[index]
+        for index, piece in self.squares.items():
             if piece in FEN_WHITE_PIECES:
                 pcolor = self.whitecolor
             elif piece in FEN_BLACK_PIECES:
@@ -303,9 +302,8 @@ class Board(Bindings):
         squares = self.squares
         occupied = list(squares.keys())
         squares.clear()
-        square_properties = Squares.squares
         for square, piece in board.items():
-            squares[square_properties[square].number] = piece.name
+            squares[fen_squares[square].number] = piece.name
         for square in occupied:
             if square not in squares:
                 squares[square] = NOPIECE
