@@ -1,4 +1,4 @@
-# chess_ui.py
+# _chess_ui.py
 # Copyright 2015 Roger Marsh
 # Licence: See LICENCE (BSD licence)
 
@@ -33,24 +33,24 @@ from solentware_bind.gui.bindings import Bindings
 
 from .gamerow import chess_db_row_game
 from .positionrow import chess_db_row_position
-from .cqlrow import chess_db_row_cql
 from .repertoirerow import chess_db_row_repertoire
+from .cqlrow import chess_db_row_cql
 from .gamegrid import (
     TagRosterGrid,
     GamePositionGames,
     PartialPositionGames,
 )
+from .cqlgrid import CQLGrid
 from .repertoiregrid import (
     RepertoireGrid,
     RepertoirePositionGames,
 )
 from .game import Game
 from .board import Board
-from .cqlgrid import CQLGrid
 from . import constants, fonts
 from ..core.filespec import (
     GAMES_FILE_DEF,
-    PARTIAL_FILE_DEF,
+    CQL_FILE_DEF,
     REPERTOIRE_FILE_DEF,
     ANALYSIS_FILE_DEF,
     VARIATION_FIELD_DEF,
@@ -64,13 +64,14 @@ from ..core.filespec import (
     BLACK_FIELD_DEF,
     RESULT_FIELD_DEF,
     OPENING_FIELD_DEF,
-    PARTIALPOSITION_NAME_FIELD_DEF,
+    QUERY_NAME_FIELD_DEF,
 )
 from .displayitems import DisplayItems
 from ..core.chessrecord import ChessDBrecordAnalysis
 from .querygrid import QueryGrid
 from .queryrow import chess_db_row_query
-from .score import ScoreNoGameException
+from .score import ScoreNoGameException, ScoreNoInitialPositionException
+from ..core import utilities
 
 
 class ChessUIError(Exception):
@@ -110,6 +111,9 @@ class ChessUI(Bindings):
         self.show_analysis = True
         self.visible_scrollbars = True
 
+        # {[%...]} comments are not suppressed by default.
+        self.suppress_structured_comment = False
+
         # Create widgets for toolbarframe (to left of Statusbar set by Chess).
         if toolbarframe is not None:
             self.tb_entry = tkinter.ttk.Entry(toolbarframe, width=20)
@@ -227,41 +231,41 @@ class ChessUI(Bindings):
         self.pw_parent_map[self.games_pw] = self.list_games_pw
         self.pw_parent_map[self.position_games_pw] = self.list_games_pw
         self.pw_parent_map[self.repertoires_pw] = self.repertoires_display_pw
-        self.pw_parent_map[
-            self.position_repertoires_pw
-        ] = self.repertoires_display_pw
+        self.pw_parent_map[self.position_repertoires_pw] = (
+            self.repertoires_display_pw
+        )
         self.pw_parent_map[self.partials_pw] = self.partials_display_pw
-        self.pw_parent_map[
-            self.position_partials_pw
-        ] = self.partials_display_pw
+        self.pw_parent_map[self.position_partials_pw] = (
+            self.partials_display_pw
+        )
         self.pw_parent_map[self.selection_rules_pw] = self.selections_pw
         self.pw_parent_map[self.view_selection_rules_pw] = self.selections_pw
 
         # These two, with self.view_games.pw bound earlier, contain the widgets
-        # used to view the detail of games, repertoires, and partial positions.
+        # used to view the detail of games, repertoires, and CQL queries.
         self.view_repertoires_pw = tkinter.ttk.PanedWindow(
             self.repertoires_display_pw, orient=tkinter.VERTICAL
         )
         self.view_partials_pw = tkinter.ttk.PanedWindow(
             self.partials_display_pw, orient=tkinter.VERTICAL
         )
-        self.pw_parent_map[
-            self.view_repertoires_pw
-        ] = self.repertoires_display_pw
+        self.pw_parent_map[self.view_repertoires_pw] = (
+            self.repertoires_display_pw
+        )
         self.pw_parent_map[self.view_partials_pw] = self.partials_display_pw
 
-        # These display the lists of games, repertoires, partial positions, and
+        # These display the lists of games, repertoires, CQL queries, and
         # game selection rules, on the database.
-        self.base_games = TagRosterGrid(self)
+        self.base_games = self._create_tag_roster_grid()
         self.base_repertoires = RepertoireGrid(self)
-        self.base_partials = CQLGrid(self)
+        self.base_partials = self._create_cql_grid()
         self.base_selections = QueryGrid(self)
 
-        # These display the lists of games, repertoires, and partial positions,
+        # These display the lists of games, repertoires, and CQL queries,
         # which match the current position of the active displayed item.
-        self.game_games = GamePositionGames(self)
+        self.game_games = self._create_game_position_games_grid()
         self.repertoire_games = RepertoirePositionGames(self)
-        self.partial_games = PartialPositionGames(self)
+        self.partial_games = self._create_partial_position_games_grid()
 
         def add_panedwindow(widget, weight):
             self.pw_current_weights[widget] = 0
@@ -310,7 +314,7 @@ class ChessUI(Bindings):
         self.partial_items = DisplayItems()
         self.selection_items = DisplayItems()
 
-        # The visibility of the repertoire and partial position lists when a
+        # The visibility of the repertoire and CQL query lists when a
         # close action was done to a database: restore to this state on opening
         # a database.
         # Anything more complex will need something like DisplayItems.
@@ -364,7 +368,7 @@ class ChessUI(Bindings):
         map_payload(self.base_selections, self.selection_rules_pw)
         map_payload(self.selection_items, self.view_selection_rules_pw)
 
-        # The game, repertoire, partial, and selection rule, widgets in their
+        # The game, repertoire, CQL query, and selection rule, widgets in their
         # own Toplevel.
         # Should these, added later, be helper class responsibility as well?
         # Similar construct in .uci.UCI too.
@@ -397,9 +401,24 @@ class ChessUI(Bindings):
 
     def add_game_to_display(self, item):
         """Add game item to GUI."""
+        if utilities.is_game_import_in_progress_txn(
+            item.ui.database, item.sourceobject
+        ):
+            tkinter.messagebox.showinfo(
+                parent=self.get_toplevel(),
+                title="Game import not complete",
+                message="".join(
+                    (
+                        "The selected game will be displayed but attempts ",
+                        "to delete, or save edits to, the game will be ",
+                        "rejected because some stages of importing this ",
+                        "game have not been completed.",
+                    )
+                ),
+            )
+            item.blockchange = True
         self.game_items.add_item_to_display(item)
         if self.game_items.contains_one_item():
-
             # Call may not be necessary.
             # This one repeats call in gamedisplay.GameDisplay.
             # item.set_score_pointer_widget_navigation_bindings(True)
@@ -411,7 +430,7 @@ class ChessUI(Bindings):
         self._show_game_games()
 
     def add_partial_position_to_display(self, item):
-        """Add partial position item to GUI."""
+        """Add CQL query item to GUI."""
         self.partial_items.add_item_to_display(item)
         if self.partial_items.contains_one_item():
             self._set_partial_position_data_source()
@@ -429,7 +448,6 @@ class ChessUI(Bindings):
         """Add repertoire item to GUI."""
         self.repertoire_items.add_item_to_display(item)
         if self.repertoire_items.contains_one_item():
-
             # Call may not be necessary.
             # item.set_score_pointer_widget_navigation_bindings(True)
 
@@ -446,7 +464,6 @@ class ChessUI(Bindings):
         """Add selection rule item to GUI."""
         self.selection_items.add_item_to_display(item)
         if self.selection_items.contains_one_item():
-
             # Call may not be necessary.
             item.set_score_pointer_widget_navigation_bindings(True)
 
@@ -463,7 +480,7 @@ class ChessUI(Bindings):
         self.game_items.configure_items_grid(self.view_games_pw)
 
     def configure_partial_grid(self):
-        """Adjust partial grid row sizes after navigate add or delete."""
+        """Adjust CQL query grid row sizes after navigate add or delete."""
         # If the constant is 2, like in configure_game_grid, sizing enters an
         # infinite loop if the first Partial Position is added when one or
         # more Repertoires are present.  The effect changes with constant, but
@@ -483,7 +500,7 @@ class ChessUI(Bindings):
         #    active_weight = 2
         # else:
         #    active_weight = 1
-        # Back to original code when repertoire and partial position fully
+        # Back to original code when repertoire and CQL query fully
         # separated.
         # The baffling problem may have been a consequence of references to
         # items displayed in list_games_pw (left-most) when deciding what to do
@@ -551,14 +568,13 @@ class ChessUI(Bindings):
                     self.base_partials.frame.focus_set()
                     return
             if self.partial_items.count_items_in_stack() != 0:
-
                 # BUG start
-                # partial position not resized
+                # CQL query not resized
                 self.partial_items.active_item.takefocus_widget.focus_set()
                 # BUG end
 
     def delete_position_view(self, item):
-        """Remove (partial) panel from GUI on 'delete' event from panel."""
+        """Remove (CQL query) panel from GUI on 'delete' event from panel."""
         views = self.partial_items
         grid_item = views.delete_item_counters(item)
         if grid_item:
@@ -648,13 +664,13 @@ class ChessUI(Bindings):
             # Show all games in base_games.
 
     def set_find_partial_name_games(self, index):
-        """Set status text to partial position name being searched."""
+        """Set status text to CQL query name being searched."""
         self.statusbar.set_status_text(
             self.partial_items.get_stack_item(index).get_text_for_statusbar()
         )
 
     def set_partial_name(self):
-        """Set status text to active partial position name."""
+        """Set status text to active CQL query name."""
         self.statusbar.set_status_text(
             self.partial_items.active_item.get_text_for_statusbar()
         )
@@ -727,7 +743,7 @@ class ChessUI(Bindings):
         self._reset_focus_on_hide_widget()
 
     def _hide_partial_position_games(self):
-        """Hide widgets containing selected partial positions and games."""
+        """Hide widgets containing selected CQL queries and games."""
         if self.partial_items.count_items_in_stack():
             return
 
@@ -746,7 +762,7 @@ class ChessUI(Bindings):
         self._reset_focus_on_hide_widget()
 
     def _hide_base_partials(self):
-        """Hide widget containing list of partial positions in database."""
+        """Hide widget containing list of CQL queries in database."""
         if not self.base_partials.is_visible():
             return
         self.base_partials.forget_payload(
@@ -756,7 +772,7 @@ class ChessUI(Bindings):
         self.base_partials.set_data_source()
 
     def hide_partial_position_grid(self):
-        """Hide widget containing list of partial positions in database."""
+        """Hide widget containing list of CQL queries in database."""
         if not self.base_partials.is_visible():
             return
         self._hide_base_partials()
@@ -980,13 +996,13 @@ class ChessUI(Bindings):
         )
 
     def _set_partial_change_notifications(self, widget, callback=None):
-        """Set or unset partial position update notifications to widget."""
+        """Set or unset CQL query update notifications to widget."""
         self.set_data_change_notifications(
             widget, (self.base_partials,), callback
         )
 
     def _set_repertoire_change_notifications(self, widget, callback=None):
-        """Set or unset partial position update notifications to widget."""
+        """Set or unset CQL query update notifications to widget."""
         self.set_data_change_notifications(
             widget, (self.base_repertoires,), callback
         )
@@ -1044,7 +1060,7 @@ class ChessUI(Bindings):
             self.base_games.set_selection_text()
 
     def set_focus_partial_game_grid(self, event=None):
-        """Give widget displaying list of games for partial position focus."""
+        """Give widget displaying list of games for CQL query focus."""
         del event
         if self.database is not None:
             self.set_toolbarframe_disabled()
@@ -1052,7 +1068,7 @@ class ChessUI(Bindings):
             self.partial_games.set_selection_text()
 
     def set_focus_partial_grid(self, event=None):
-        """Give widget displaying list of partial positions the focus."""
+        """Give widget displaying list of CQL queries the focus."""
         del event
         if self.database is not None:
             self.set_toolbarframe_normal(
@@ -1062,7 +1078,7 @@ class ChessUI(Bindings):
             self.base_partials.set_selection_text()
 
     def set_focus_partialpanel_item(self, event=None):
-        """Give partial position at top of stack the focus."""
+        """Give CQL query at top of stack the focus."""
         del event
         self.set_toolbarframe_disabled()
         self.partial_items.set_focus()
@@ -1129,7 +1145,7 @@ class ChessUI(Bindings):
         self.game_games.set_data_source(sqds)
 
     def _set_partial_position_data_source(self):
-        """Attach database to widget displaying games for partial position."""
+        """Attach database to widget displaying games for CQL query."""
         if self.database is not None:
             sqds = self.partialpositionds(
                 self.database,
@@ -1143,7 +1159,7 @@ class ChessUI(Bindings):
         # The expected effect, using correct header, does not happen.
         # self.partial_games.set_data_header(header=ChessDBrowGame)
         # In some runs always does right thing but in others gives empty grid.
-        # Seems to be wrong always if partial item is asked for first.
+        # Seems to be wrong always if CQL query item is asked for first.
         # self.partial_games.make_header(ChessDBrowGame.header_specification)
         self.partial_games.set_data_source(sqds)
 
@@ -1171,7 +1187,6 @@ class ChessUI(Bindings):
     def _show_base_games(self):
         """Show widget containing list of games in database."""
         if self.list_games_pw.pane(self.games_pw, "weight"):
-
             # Size is assumed to be (1,1) as created at first call and grid
             # will be filled after <Configure> event to size the widget.
             # Subsequent calls are not associated with a <Configure> event so
@@ -1201,7 +1216,14 @@ class ChessUI(Bindings):
             self.game_items.set_insert_or_delete_on_all_items()
         if self.partial_items.count_items_in_stack():
             self._set_partial_position_data_source()
-            self.partial_items.active_item.refresh_game_list()
+            sourceobject = self.partial_items.active_item.sourceobject
+            self.partial_items.active_item.refresh_game_list(
+                key_recno=(
+                    sourceobject.key.recno
+                    if sourceobject is not None
+                    else None
+                )
+            )
             self.partial_items.set_insert_or_delete_on_all_items()
         if self.repertoire_items.count_items_in_stack():
             self._set_repertoire_data_source()
@@ -1273,7 +1295,7 @@ class ChessUI(Bindings):
         self._configure_panedwindows()
 
     def _show_partial_position_games(self):
-        """List games with a position fitting the active partial position."""
+        """List games with a position fitting the active CQL query."""
         if not self.partial_items.count_items_in_stack():
             if not self.top_pw.pane(self.analysis_pw, "weight"):
                 return
@@ -1330,15 +1352,15 @@ class ChessUI(Bindings):
         self._configure_panedwindows()
 
     def _create_partial_position_datasource(self, database):
-        """Create a new DataSource for list of partial positions."""
+        """Create a new DataSource for list of CQL queries."""
         # Probably wrong because datasource = F(active item), but this is a
         # constant at present.
         self.base_partials.set_data_source(
             DataSource(
                 database,
-                PARTIAL_FILE_DEF,
-                PARTIALPOSITION_NAME_FIELD_DEF,
-                chess_db_row_cql(self),
+                CQL_FILE_DEF,
+                QUERY_NAME_FIELD_DEF,
+                self._create_partial_position_row(),
             ),
             self.base_partials.on_data_change,
         )
@@ -1346,10 +1368,13 @@ class ChessUI(Bindings):
             self._set_partial_position_data_source()
             self.partial_items.set_insert_or_delete_on_all_items()
 
-    def _show_base_partials(self):
-        """Show widget containing list of partial positions in database."""
-        if self.partials_display_pw.pane(self.partials_pw, "weight"):
+    def _create_partial_position_row(self):
+        """Return ChessDBrowCQL instance."""
+        return chess_db_row_cql(self)
 
+    def _show_base_partials(self):
+        """Show widget containing list of CQL queries in database."""
+        if self.partials_display_pw.pane(self.partials_pw, "weight"):
             # Size is assumed to be (1,1) as created at first call and grid
             # will be filled after <Configure> event to size the widget.
             # Subsequent calls are not associated with a <Configure> event so
@@ -1359,7 +1384,7 @@ class ChessUI(Bindings):
                 self.partials_pw.after_idle(self.base_partials.load_new_index)
 
     def show_partial_position_grid(self, database):
-        """Show widget containing list of partial positions in database."""
+        """Show widget containing list of CQL queries in database."""
         self._create_partial_position_datasource(database)
 
         # # Hack to cope with GUI use while Import in progress.
@@ -1391,7 +1416,6 @@ class ChessUI(Bindings):
     def _show_base_repertoires(self):
         """Show widget containing list of repertoire games in database."""
         if self.repertoires_display_pw.pane(self.repertoires_pw, "weight"):
-
             # Size is assumed to be (1,1) as created at first call and grid
             # will be filled after <Configure> event to size the widget.
             # Subsequent calls are not associated with a <Configure> event so
@@ -1428,13 +1452,13 @@ class ChessUI(Bindings):
 
     def get_import_subprocess_poll(self):
         """Poll the import subprocess and return the response."""
-        return self._import_subprocess.poll()
+        return self._import_subprocess.is_alive()
 
     def is_import_subprocess_active(self):
         """Return True if the import subprocess object is active."""
         if self._import_subprocess is None:
             return False
-        return self._import_subprocess.poll() is None
+        return self._import_subprocess.is_alive()
 
     def get_toplevel(self):
         """Return the toplevel widget."""
@@ -1603,7 +1627,10 @@ class ChessUI(Bindings):
                     if item.current is None:
                         try:
                             position = item.fen_tag_tuple_square_piece_map()
-                        except ScoreNoGameException:
+                        except (
+                            ScoreNoGameException,
+                            ScoreNoInitialPositionException,
+                        ):
                             continue
                     else:
                         position = item.tagpositionmap[item.current]
@@ -1780,12 +1807,7 @@ class ChessUI(Bindings):
 
     def make_position_analysis_data_source(self):
         """Create a new DataSource for stored chess engine analysis."""
-        if self.database:
-
-            # Without the 'is not None' seems unreliable at 08 Nov 2015.
-            # What is wrong with 'if <obj>:' where obj is a bsddb3.DB instance?
-            # It does work sometimes, so some environment clutter perhaps.
-            # Not yet known what happens with sqlite3 and so forth.
+        if self.database is not None and self.database.dbenv is not None:
             if (
                 self.database.is_database_file_active(ANALYSIS_FILE_DEF)
                 is not None
@@ -1817,7 +1839,6 @@ class ChessUI(Bindings):
     def _show_base_selection_rules(self):
         """Show widget containing list of selection rules in database."""
         if self.selections_pw.pane(self.selection_rules_pw, "weight"):
-
             # Size is assumed to be (1,1) as created at first call and grid
             # will be filled after <Configure> event to size the widget.
             # Subsequent calls are not associated with a <Configure> event so
@@ -1997,3 +2018,35 @@ class ChessUI(Bindings):
                     )
                 ),
             )
+
+    def _create_tag_roster_grid(self):
+        """Return TagRosterGrid instance."""
+        return TagRosterGrid(self)
+
+    def _create_game_position_games_grid(self):
+        """Return GamePositionGames instance."""
+        return GamePositionGames(self)
+
+    def _create_partial_position_games_grid(self):
+        """Return PartialPositionGames instance."""
+        return PartialPositionGames(self)
+
+    def _create_cql_grid(self):
+        """Return CQLGrid instance."""
+        return CQLGrid(self)
+
+    def is_database_access_inhibited(self):
+        """Return True if database cannot be accessed."""
+        if self.database is None or self.database.dbenv is None:
+            return True
+        return False
+
+    def is_database_update_inhibited(self):
+        """Return True if database cannot be updated."""
+        if self.is_database_access_inhibited():
+            return True
+        if self.database.any_cql_queries_pending_evaluation():
+            return True
+        if utilities.is_import_in_progress_txn(self.database):
+            return True
+        return False
