@@ -9,20 +9,21 @@ import tkinter.messagebox
 from solentware_grid.datagrid import DataGrid
 
 from ..core.chessrecord import ChessDBrecordPartial
-from .cqldisplay import CQLDisplay, CQLDisplayEdit
-from .cqlrow import ChessDBrowCQL
-from ..core import export_chessql
 from .eventspec import EventSpec
+from ..core import export_chessql
 from .display import Display
 from ..shared.cql_gamelist_query import CQLGameListQuery
 from ..shared.allgrid import AllGrid
+from .cqlrow import ChessDBrowCQL
+from .cqlupdate import CQLUpdate
+from .cqldelete import CQLDelete
+from ..cql.queryevaluator import QueryEvaluatorError
 
 
 class CQLListGrid(AllGrid, CQLGameListQuery, DataGrid, Display):
     """A DataGrid for lists of ChessQL statements.
 
     Subclasses provide navigation and extra methods appropriate to list use.
-
     """
 
     def __init__(self, parent, ui):
@@ -42,6 +43,11 @@ class CQLListGrid(AllGrid, CQLGameListQuery, DataGrid, Display):
         # passed to CQLDisplay. (Needs 'import Tkinter' above.)
         # Rather than passing the container where the Frame created by
         # CQLDisplay is to be put.
+        if self.ui.is_database_access_inhibited():
+            self._database_not_available_dialogue("Display")
+            return None
+        if self.ui.is_database_update_inhibited():
+            self._database_update_not_available_dialogue("Display")
         selection = self.make_display_widget(selected)
         self.ui.add_partial_position_to_display(selection)
         self.ui.partial_items.increment_object_count(key)
@@ -50,27 +56,29 @@ class CQLListGrid(AllGrid, CQLGameListQuery, DataGrid, Display):
         return selection
 
     def make_display_widget(self, sourceobject):
-        """Return a CQLDisplay for sourceobject."""
-        selection = CQLDisplay(
+        """Return a CQLDelete instance."""
+        selection = CQLDelete(
             master=self.ui.view_partials_pw,
             ui=self.ui,
             items_manager=self.ui.partial_items,
             itemgrid=self.ui.partial_games,
             sourceobject=sourceobject,
         )
-        #    sourceobject.get_srvalue())
-        selection.cql_statement.process_statement(sourceobject.get_srvalue())
+        selection.cql_statement.split_statement(sourceobject.get_srvalue())
         return selection
 
     def _edit_selected_item(self, key):
-        """Create a CQLDisplayEdit for ChessQL statement."""
+        """Create a CQLUpdate for ChessQL statement."""
         selected = self.get_visible_record(key)
         if selected is None:
             return None
         # Should the Frame containing board and position be created here and
-        # passed to CQLDisplayEdit. (Which needs 'import Tkinter' above.)
+        # passed to CQLUpdate. (Which needs 'import Tkinter' above.)
         # Rather than passing the container where the Frame created by
-        # CQLDisplayEdit is to be put.
+        # CQLUpdate is to be put.
+        if self.ui.is_database_update_inhibited():
+            self._database_not_available_dialogue("Display Edit")
+            return None
         selection = self.make_edit_widget(selected)
         self.ui.add_partial_position_to_display(selection)
         self.ui.partial_items.increment_object_count(key)
@@ -79,16 +87,15 @@ class CQLListGrid(AllGrid, CQLGameListQuery, DataGrid, Display):
         return selection
 
     def make_edit_widget(self, sourceobject):
-        """Return a CQLDisplayEdit for sourceobject."""
-        selection = CQLDisplayEdit(
+        """Return a ..gui.cqlinsert.CQLUpdate instance."""
+        selection = CQLUpdate(
             master=self.ui.view_partials_pw,
             ui=self.ui,
             items_manager=self.ui.partial_items,
             itemgrid=self.ui.partial_games,
             sourceobject=sourceobject,
         )
-        #    sourceobject.get_srvalue())
-        selection.cql_statement.process_statement(sourceobject.get_srvalue())
+        selection.cql_statement.split_statement(sourceobject.get_srvalue())
         return selection
 
     def set_properties(self, key, dodefaultaction=True):
@@ -118,71 +125,178 @@ class CQLListGrid(AllGrid, CQLGameListQuery, DataGrid, Display):
 
     def launch_delete_record(self, key, modal=True):
         """Create delete dialogue."""
+        if self.ui.is_database_update_inhibited():
+            self._database_not_available_dialogue("Delete")
+            return
         oldobject = ChessDBrecordPartial()
         oldobject.load_record(
             (self.objects[key].key.pack(), self.objects[key].srvalue)
         )
-        self.create_delete_dialog(
-            self.objects[key],
-            oldobject,
-            modal,
-            title="Delete ChessQL Statement",
-        )
+        # A "" srvalue would be assumed an insert action later.
+        if not self.objects[key].srvalue:
+            tkinter.messagebox.showinfo(
+                title="Delete ChessQL Statement",
+                message="Empty statements are rejected by CQL",
+            )
+            return
+        try:
+            self.create_delete_dialog(
+                self.objects[key],
+                oldobject,
+                modal,
+                title="Delete ChessQL Statement",
+            )
+        except QueryEvaluatorError as exc:
+            tkinter.messagebox.showinfo(
+                title="Delete ChessQL Statement",
+                message="".join(
+                    (
+                        "Attempt to start Delete action fails\n\nThe ",
+                        "reported exception is\n\n",
+                        str(exc),
+                    )
+                ),
+            )
 
     def launch_edit_record(self, key, modal=True):
         """Create edit dialogue."""
-        self.create_edit_dialog(
-            self.objects[key],
-            ChessDBrecordPartial(),
-            ChessDBrecordPartial(),
-            False,
-            modal,
-            title="Edit ChessQL Statement",
-        )
+        if self.ui.is_database_update_inhibited():
+            self._database_not_available_dialogue("Display Edit")
+            return
+        # A "" srvalue would be assumed an insert action later.
+        if not self.objects[key].srvalue:
+            tkinter.messagebox.showinfo(
+                title="Edit ChessQL Statement",
+                message="Empty statements are rejected by CQL",
+            )
+            return
+        try:
+            self.create_edit_dialog(
+                self.objects[key],
+                ChessDBrecordPartial(),
+                ChessDBrecordPartial(),
+                False,
+                modal,
+                title="Edit ChessQL Statement",
+            )
+        except QueryEvaluatorError as exc:
+            tkinter.messagebox.showinfo(
+                title="Edit ChessQL Statement",
+                message="".join(
+                    (
+                        "Attempt to start Edit action fails\n\nThe ",
+                        "reported exception is\n\n",
+                        str(exc),
+                    )
+                ),
+            )
 
     def launch_edit_show_record(self, key, modal=True):
         """Create edit dialogue including reference copy of original."""
-        self.create_edit_dialog(
-            self.objects[key],
-            ChessDBrecordPartial(),
-            ChessDBrecordPartial(),
-            True,
-            modal,
-            title="Edit ChessQL Statement",
-        )
+        if self.ui.is_database_update_inhibited():
+            self._database_not_available_dialogue("Display Edit")
+            return
+        # A "" srvalue would be assumed an insert action later.
+        if not self.objects[key].srvalue:
+            tkinter.messagebox.showinfo(
+                title="Edit ChessQL Statement",
+                message="Empty statements are rejected by CQL",
+            )
+            return
+        try:
+            self.create_edit_dialog(
+                self.objects[key],
+                ChessDBrecordPartial(),
+                ChessDBrecordPartial(),
+                True,
+                modal,
+                title="Edit ChessQL Statement",
+            )
+        except QueryEvaluatorError as exc:
+            tkinter.messagebox.showinfo(
+                title="Edit ChessQL Statement",
+                message="".join(
+                    (
+                        "Attempt to start Edit action fails\n\nThe ",
+                        "reported exception is\n\n",
+                        str(exc),
+                    )
+                ),
+            )
 
     def launch_insert_new_record(self, modal=True):
         """Create insert dialogue."""
+        if self.ui.is_database_update_inhibited():
+            self._database_not_available_dialogue("Insert")
+            return
         instance = self.datasource.new_row()
 
-        # Later process_statement() causes display of empty title and
-        # query lines.
+        # Later prepare_cql_statement() causes display of empty title and
+        # query lines by assuming "" implies insert action without passing
+        # query to CQL which would reject it.
         instance.srvalue = repr("")
 
-        self.create_edit_dialog(
-            instance,
-            ChessDBrecordPartial(),
-            None,
-            False,
-            modal,
-            title="New ChessQL Statement",
-        )
+        try:
+            self.create_edit_dialog(
+                instance,
+                ChessDBrecordPartial(),
+                None,
+                False,
+                modal,
+                title="New ChessQL Statement",
+            )
+        except QueryEvaluatorError as exc:
+            tkinter.messagebox.showinfo(
+                title="New ChessQL Statement",
+                message="".join(
+                    (
+                        "Attempt to start Insert action fails\n\nThe ",
+                        "reported exception is\n\n",
+                        str(exc),
+                    )
+                ),
+            )
 
     def launch_show_record(self, key, modal=True):
         """Create show dialogue."""
+        if self.ui.is_database_access_inhibited():
+            self._database_not_available_dialogue("Show")
+            return
         oldobject = ChessDBrecordPartial()
         oldobject.load_record(
             (self.objects[key].key.pack(), self.objects[key].srvalue)
         )
-        self.create_show_dialog(
-            self.objects[key], oldobject, modal, title="Show ChessQL Statement"
-        )
+        # A "" srvalue would be assumed an insert action later.
+        if not self.objects[key].srvalue:
+            tkinter.messagebox.showinfo(
+                title="Show ChessQL Statement",
+                message="Empty statements are rejected by CQL",
+            )
+            return
+        try:
+            self.create_show_dialog(
+                self.objects[key],
+                oldobject,
+                modal,
+                title="Show ChessQL Statement",
+            )
+        except QueryEvaluatorError as exc:
+            tkinter.messagebox.showinfo(
+                title="Show ChessQL Statement",
+                message="".join(
+                    (
+                        "Attempt to start Show action fails\n\nThe ",
+                        "reported exception is\n\n",
+                        str(exc),
+                    )
+                ),
+            )
 
     def _export_partial(self, event=None):
-        """Export selected partial position definitions."""
+        """Export selected CQL query definitions."""
         del event
         export_chessql.export_selected_positions(
-            self, self.ui.get_export_filename("Partial Positions", pgn=False)
+            self, self.ui.get_export_filename("CQL Queries", pgn=False)
         )
 
 
@@ -198,6 +312,27 @@ class CQLGrid(CQLListGrid):
         super().__init__(ui.partials_pw, ui)
         self.make_header(ChessDBrowCQL.header_specification)
         self.__bind_on()
+        self._set_initial_bindings()
+
+    def bind_off(self):
+        """Disable all bindings."""
+        super().bind_off()
+        self._set_bindings_in_cqlgrid_bind_off()
+
+    def bind_on(self):
+        """Enable all bindings."""
+        super().bind_on()
+        self.__bind_on()
+
+    def __bind_on(self):
+        """Enable all bindings."""
+        self._set_bindings_in_cqlgrid___bind_on()
+
+    def _set_initial_bindings(self):
+        """Set the event bindings in __init__() method."""
+        # This method has code moved from .cqlgrid.CQLGrid.__init__()
+        # so it can be used by CQLGrid.__init__() in other packages.
+        # pylint: disable=no-member
         self._set_popup_bindings(
             self.menupopup,
             (
@@ -258,9 +393,11 @@ class CQLGrid(CQLListGrid):
             "Navigation", self.menupopupnorow, bindings
         )
 
-    def bind_off(self):
-        """Disable all bindings."""
-        super().bind_off()
+    def _set_bindings_in_cqlgrid_bind_off(self):
+        """Set the event bindings in bind_off() method."""
+        # This method has code moved from .cqlgrid.CQLGrid.bind_off()
+        # so it can be used by CQLGrid.bind_off() in other packages.
+        # pylint: disable=no-member
         self._set_event_bindings_frame(
             (
                 (EventSpec.navigate_to_active_partial, ""),
@@ -282,13 +419,11 @@ class CQLGrid(CQLListGrid):
             )
         )
 
-    def bind_on(self):
-        """Enable all bindings."""
-        super().bind_on()
-        self.__bind_on()
-
-    def __bind_on(self):
-        """Enable all bindings."""
+    def _set_bindings_in_cqlgrid___bind_on(self):
+        """Set the event bindings in __bind_on() method."""
+        # This method has code moved from .cqlgrid.CQLGrid.__bind_on()
+        # so it can be used by CQLGrid.__bind_on() in other packages.
+        # pylint: disable=no-member
         self._set_event_bindings_frame(
             (
                 (
@@ -344,6 +479,9 @@ class CQLGrid(CQLListGrid):
         message display.
 
         """
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         del event
         if not self.get_visible_selected_key():
             return
@@ -361,6 +499,9 @@ class CQLGrid(CQLListGrid):
         message display.
 
         """
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         del event
         self._set_find_cql_statement_name_games(self.pointer_popup_selection)
         self.frame.after_idle(
@@ -375,6 +516,9 @@ class CQLGrid(CQLListGrid):
         Call from _display_cql_statement only.
 
         """
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         self._display_selected_item(self.get_visible_selected_key())
         self.cancel_selection()
 
@@ -384,6 +528,9 @@ class CQLGrid(CQLListGrid):
         Call from _display_cql_statement_from_popup only.
 
         """
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         self._display_selected_item(self.pointer_popup_selection)
 
     def _edit_cql_statement(self, event=None):
@@ -393,6 +540,9 @@ class CQLGrid(CQLListGrid):
         message display.
 
         """
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         del event
         if not self.get_visible_selected_key():
             return
@@ -408,6 +558,9 @@ class CQLGrid(CQLListGrid):
         message display.
 
         """
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         del event
         self._set_find_cql_statement_name_games(self.pointer_popup_selection)
         self.frame.after_idle(
@@ -422,6 +575,9 @@ class CQLGrid(CQLListGrid):
         Call from _edit_cql_statement only.
 
         """
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         self._edit_selected_item(self.get_visible_selected_key())
         self.cancel_selection()
 
@@ -431,10 +587,16 @@ class CQLGrid(CQLListGrid):
         Call from _edit_cql_statement_from_popup only.
 
         """
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         self._edit_selected_item(self.pointer_popup_selection)
 
     def _set_find_cql_statement_name_games(self, key):
         """Set status text to active ChessQL statement name."""
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         if self.ui.partial_items.count_items_in_stack():
             # do search at this time only if no ChessQL statements displayed
             return
@@ -447,15 +609,11 @@ class CQLGrid(CQLListGrid):
             )
         )
 
-    def on_partial_change(self, instance):
-        # may turn out to be just to catch datasource is None
-        """Refresh query grid after database update for instance."""
-        if self.get_data_source() is None:
-            return
-        super().on_data_change(instance)
-
     def set_selection_text(self):
         """Set status bar to display ChessQL statement name."""
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         if self.selection:
             value = self.objects[self.selection[0]].value
             self.ui.statusbar.set_status_text(
@@ -473,22 +631,34 @@ class CQLGrid(CQLListGrid):
 
     def is_visible(self):
         """Return True if list of ChessQL statements is displayed."""
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         return str(self.get_frame()) in self.ui.partials_pw.panes()
 
     def make_display_widget(self, sourceobject):
         """Return a CQLDisplay for sourceobject."""
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         selection = super().make_display_widget(sourceobject)
         selection.set_and_tag_item_text()
         return selection
 
     def make_edit_widget(self, sourceobject):
-        """Return a CQLDisplayEdit for sourceobject."""
+        """Return a CQLUpdate for sourceobject."""
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         selection = super().make_edit_widget(sourceobject)
         selection.set_and_tag_item_text(reset_undo=True)
         return selection
 
     def focus_set_frame(self, event=None):
         """Delegate to superclass then set toolbar widget states."""
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         super().focus_set_frame(event=event)
         self.ui.set_toolbarframe_normal(
             self.ui.move_to_selection, self.ui.filter_selection
@@ -502,6 +672,9 @@ class CQLGrid(CQLListGrid):
         display.
 
         """
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
         try:
             super().set_selection(key)
         except KeyError:
@@ -516,3 +689,25 @@ class CQLGrid(CQLListGrid):
                     )
                 ),
             )
+
+    def on_data_change(self, instance):
+        """Delegate to superclass by after_idle() if database is Symas LMMD.
+
+        For other database engines delegate to superclass directly.
+
+        """
+        # This method has moved from .cqlgrid.CQLGrid so it can be used
+        # by CQLGrid classes in other packages.
+        # pylint: disable=no-member
+        # Hack to prevent crash in _lmdb accessing Symas LMMD via lmdb.
+        # The crash occurred on using the 'non-F11' options to insert, edit,
+        # or delete, a ChessQL statement.
+        # Problem seems to be a read-only transaction done in refresh_widgets
+        # callbacks for chessql actions: which does not occur for other items.
+        # There is, correctly at this point, no way to determine _lmdb is in
+        # use apart from some assumption about the state of database engine.
+        # The 'after_idle' route for all database engines may be fine too.
+        if self.datasource.dbhome.dbenv.__class__.__name__ == "Environment":
+            self.parent.after_idle(super().on_data_change, *(instance,))
+        else:
+            super().on_data_change(instance)
