@@ -1,8 +1,12 @@
 # cqlrow.py
-# Copyright 2016 Roger Marsh
+# Copyright 2016, 2025 Roger Marsh
 # Licence: See LICENCE (BSD licence)
 
-"""Create widgets to display Chess Query Language (ChessQL) statement."""
+"""Create widgets to display Chess Query Language (ChessQL) statement.
+
+Adapt chesstab.gui._cqlrow.ChessDBrowCQL class to use chesstab version
+of CQLDbEdit class.
+"""
 
 import tkinter
 
@@ -16,11 +20,11 @@ from solentware_grid.gui.datarow import (
 )
 
 from ..core.chessrecord import ChessDBrecordPartial
-from .cqldbedit import CQLDbEdit
 from .cqldbdelete import CQLDbDelete
 from .cqldbshow import CQLDbShow
 from . import constants
 from ..shared.allrow import AllRow
+from .cqldbedit import CQLDbEdit
 
 
 class ChessDBrowCQL(AllRow, ChessDBrecordPartial, DataRow):
@@ -33,15 +37,15 @@ class ChessDBrowCQL(AllRow, ChessDBrecordPartial, DataRow):
     header_specification = [
         {
             WIDGET: tkinter.Label,
-            WIDGET_CONFIGURE: dict(
-                text="Description",
-                anchor=tkinter.W,
-                padx=0,
-                pady=1,
-                font="TkDefaultFont",
-            ),
-            GRID_CONFIGURE: dict(column=0, sticky=tkinter.EW),
-            GRID_COLUMNCONFIGURE: dict(weight=1, uniform="pp"),
+            WIDGET_CONFIGURE: {
+                "text": "Description",
+                "anchor": tkinter.W,
+                "padx": 0,
+                "pady": 1,
+                "font": "TkDefaultFont",
+            },
+            GRID_CONFIGURE: {"column": 0, "sticky": tkinter.EW},
+            GRID_COLUMNCONFIGURE: {"weight": 1, "uniform": "pp"},
             ROW: 0,
         },
     ]
@@ -59,13 +63,13 @@ class ChessDBrowCQL(AllRow, ChessDBrecordPartial, DataRow):
         self.row_specification = [
             {
                 WIDGET: tkinter.Label,
-                WIDGET_CONFIGURE: dict(
-                    anchor=tkinter.W,
-                    font=constants.LISTS_OF_GAMES_FONT,
-                    pady=1,
-                    padx=0,
-                ),
-                GRID_CONFIGURE: dict(column=0, sticky=tkinter.EW),
+                WIDGET_CONFIGURE: {
+                    "anchor": tkinter.W,
+                    "font": constants.LISTS_OF_GAMES_FONT,
+                    "pady": 1,
+                    "padx": 0,
+                },
+                GRID_CONFIGURE: {"column": 0, "sticky": tkinter.EW},
                 ROW: 0,
             },
         ]
@@ -124,7 +128,7 @@ def chess_db_row_cql(chessui):
 
     chessui is a chess_ui.ChessUI instance.
 
-    The returned function takes a ChessDatabase instance as it's argument.
+    The returned function takes a Database instance as it's argument.
     """
 
     def make_selection(database=None):
