@@ -49,6 +49,8 @@ from solentware_grid.gui.datadelete import RecordDelete
 from .score import NonTagBind, ScoreNoGameException
 from .scorepgn import ScorePGN
 from .displaytext import ShowText, DisplayText, EditText, InsertText
+from ..core.chessrecord import ChessDBvaluePGNDelete
+from ..core import constants
 
 
 # ShowPGN because DisplayPGN fits GameDisplay (and *Repertoire*)
@@ -144,8 +146,8 @@ class ShowPGN(ShowText, ScorePGN):
 
     # The insert_game_database method, coerced into sameness from the methods
     # in gamedisplay._GameDisplay and repertoiredisplay._RepertoireDisplay with
-    # class attibutes pgn_score_name, pgn_score_source_name, pgn_score_tags,
-    # and method mark_partial_positions_to_be_recalculated, and property
+    # class attibutes pgn_score_name, pgn_score_source, pgn_score_tags,
+    # and method mark_cql_statements_evaluated, and property
     # ui_base_table.  The clarity of both common bits and differences
     # seems to justify the extra syntactic complexity.
 
@@ -166,11 +168,18 @@ class ShowPGN(ShowText, ScorePGN):
 
         # This should see if game with same PGN Tags already exists,
         # after checking for database open, and offer option to insert anyway.
-        if self.ui.database is None:
+        if self.is_database_update_inhibited():
             tkinter.messagebox.showinfo(
                 parent=self.ui.get_toplevel(),
                 title=title,
-                message=psn.join(("Cannot add ", ":\n\nNo database open.")),
+                message="".join(
+                    (
+                        "Cannot add ",
+                        psn,
+                        "\n\nNo database open or an import or CQL query ",
+                        "is unfinished",
+                    )
+                ),
             )
             return None
 
@@ -193,46 +202,81 @@ class ShowPGN(ShowText, ScorePGN):
                 message=psn.join(("Add ", " to database abandonned.")),
             )
             return None
-        updater = self._game_updater(repr(self.score.get("1.0", tkinter.END)))
+        updater = self._game_updater(
+            repr(
+                [
+                    repr(self.score.get("1.0", tkinter.END)),
+                    {constants.FILE: "/", constants.GAME: ""},
+                ]
+            )
+        )
         if not updater.value.collected_game.is_pgn_valid():
+            msg = [
+                "Please re-confirm request to insert ",
+                psn,
+                ".",
+            ]
+            if not updater.value.collected_game.is_movetext_valid():
+                msg.extend(["\n\nErrors exist in the Movetext."])
+            if not updater.value.collected_game.is_tag_roster_valid():
+                # Get repertoire distiguished first, then figure how to
+                # implement in existing subclasses.
+                # Prevent lmdb exceptions for zero length keys.
+                if psn.lower() == "repertoire":
+                    msg = [
+                        "Cannot insert repertoire because either ",
+                        "Opening or Result is not given.",
+                    ]
+                    tkinter.messagebox.showinfo(
+                        parent=self.ui.get_toplevel(),
+                        title=title,
+                        message="".join(msg),
+                    )
+                    return None
+                msg.extend(
+                    [
+                        "\n\nEither a mandatory Tag Pair is missing,",
+                        '\n\nor a Tag Pair has value "" if this is ',
+                        "not allowed.",
+                    ]
+                )
             if tkinter.messagebox.YES != tkinter.messagebox.askquestion(
                 parent=self.ui.get_toplevel(),
                 title=title,
-                message=psn.join(
-                    (
-                        "The new ",
-                        "".join(
-                            (
-                                " score contains at least one illegal move ",
-                                "in PGN.\n\nPlease re-confirm request to ",
-                                "insert ",
-                            )
-                        ),
-                        ".",
-                    )
-                ),
+                message="".join(msg),
             ):
                 return None
-            updater.value.set_game_source(self.pgn_score_source_name)
+            updater.value.gamesource = self.pgn_score_source
         editor = RecordEdit(updater, None)
-        editor.set_data_source(datasource, editor.on_data_change)
+        editor.set_data_source(source=datasource)
         updater.set_database(editor.get_data_source().dbhome)
-        self.mark_partial_positions_to_be_recalculated(datasource=datasource)
+        self.mark_games_evaluated(datasource=datasource)
+        self.mark_all_cql_statements_not_evaluated(datasource=datasource)
         updater.key.recno = None
         editor.put()
-        tags = updater.value.collected_game.pgn_tags
-        tkinter.messagebox.showinfo(
-            parent=self.ui.get_toplevel(),
-            title=title,
-            message="".join(
-                (
-                    psn.title(),
-                    ' "',
-                    "  ".join([tags.get(k, "") for k in self.pgn_score_tags]),
-                    '" added to database.',
+        if self.valid_cql_statements_exist(datasource=datasource):
+            self.run_cql_evaluator(datasource=datasource, ui=self.ui)
+            if self.ui.partial_items.count_items_in_stack():
+                active_item = self.ui.partial_items.active_item
+                active_item.refresh_game_list(
+                    key_recno=active_item.sourceobject.key.recno
                 )
-            ),
-        )
+        else:
+            tags = updater.value.collected_game.pgn_tags
+            tkinter.messagebox.showinfo(
+                parent=self.ui.get_toplevel(),
+                title=title,
+                message="".join(
+                    (
+                        psn.title(),
+                        ' "',
+                        "  ".join(
+                            [tags.get(k, "") for k in self.pgn_score_tags]
+                        ),
+                        '" added to database.',
+                    )
+                ),
+            )
         return True
 
 
@@ -244,11 +288,18 @@ class DisplayPGN(DisplayText):
         del event
         title = " ".join(("Delete", self.pgn_score_name.title()))
         psn = self.pgn_score_name
-        if self.ui.database is None:
+        if self.is_database_update_inhibited():
             tkinter.messagebox.showinfo(
                 parent=self.ui.get_toplevel(),
                 title=title,
-                message=psn.join(("Cannot delete ", ":\n\nNo database open.")),
+                message="".join(
+                    (
+                        "Cannot delete ",
+                        psn,
+                        "\n\nNo database open or an import or CQL query ",
+                        "is unfinished",
+                    )
+                ),
             )
             return
         datasource = self.ui_base_table.get_data_source()
@@ -299,15 +350,20 @@ class DisplayPGN(DisplayText):
             message=psn.join(("Confirm request to delete ", " from database")),
         ):
             return
-        original = self.pgn_score_updater()
+        original = self.pgn_score_updater(valueclass=ChessDBvaluePGNDelete)
         original.load_record(
             (self.sourceobject.key.recno, self.sourceobject.srvalue)
         )
         self.pgn_score_original_value(original.value)
         editor = RecordDelete(original)
-        editor.set_data_source(datasource, editor.on_data_change)
-        self.mark_partial_positions_to_be_recalculated(datasource=datasource)
+        editor.set_data_source(source=datasource)
+        self.mark_games_evaluated(datasource=datasource, allexcept=original)
+        self.mark_all_cql_statements_not_evaluated(datasource=datasource)
+        self.remove_game_key_from_all_cql_query_match_lists(
+            datasource=datasource, gamekey=self.sourceobject.key.recno
+        )
         editor.delete()
+        self.clear_cql_queries_pending_evaluation(datasource=datasource)
         tags = original.value.collected_game.pgn_tags
         tkinter.messagebox.showinfo(
             parent=self.ui.get_toplevel(),
@@ -415,11 +471,18 @@ class EditPGN(EditText):
         del event
         title = " ".join(("Edit", self.pgn_score_name.title()))
         psn = self.pgn_score_name
-        if self.ui.database is None:
+        if self.is_database_update_inhibited():
             tkinter.messagebox.showinfo(
                 parent=self.ui.get_toplevel(),
                 title=title,
-                message=psn.join(("Cannot edit ", ":\n\nNo database open.")),
+                message="".join(
+                    (
+                        "Cannot edit ",
+                        psn,
+                        "\n\nNo database open or an import or CQL query ",
+                        "is unfinished",
+                    )
+                ),
             )
             return
         datasource = self.ui_base_table.get_data_source()
@@ -470,7 +533,7 @@ class EditPGN(EditText):
             message=psn.join(("Confirm request to edit ", ".")),
         ):
             return
-        original = self.pgn_score_updater()
+        original = self.pgn_score_updater(valueclass=ChessDBvaluePGNDelete)
         original.load_record(
             (self.sourceobject.key.recno, self.sourceobject.srvalue)
         )
@@ -479,49 +542,76 @@ class EditPGN(EditText):
         # is it better to use DataClient directly?
         # Then original would not be used. Instead DataSource.new_row
         # gets record keyed by sourceobject and update is used to edit this.
-        text = self.get_score_error_escapes_removed()
-        updater = self._game_updater(repr(text))
+        updater = self._game_updater(self._construct_record_value())
         editor = RecordEdit(updater, original)
-        editor.set_data_source(datasource, editor.on_data_change)
+        editor.set_data_source(source=datasource)
         updater.set_database(editor.get_data_source().dbhome)
         if not updater.value.collected_game.is_pgn_valid():
+            msg = [
+                "Please re-confirm request to edit ",
+                psn,
+                ".",
+            ]
+            if not updater.value.collected_game.is_movetext_valid():
+                msg.extend(["\n\nErrors exist in the Movetext."])
+            if not updater.value.collected_game.is_tag_roster_valid():
+                # Get repertoire distiguished first, then figure how to
+                # implement in existing subclasses.
+                # Prevent lmdb exceptions for zero length keys.
+                if psn.lower() == "repertoire":
+                    msg = [
+                        "Cannot edit repertoire because either ",
+                        "Opening or Result is not given.",
+                    ]
+                    tkinter.messagebox.showinfo(
+                        parent=self.ui.get_toplevel(),
+                        title=title,
+                        message="".join(msg),
+                    )
+                    return
+                msg.extend(
+                    [
+                        "\n\nEither a mandatory Tag Pair is missing,",
+                        '\n\nor a Tag Pair has value "" if this is ',
+                        "not allowed.",
+                    ]
+                )
             if tkinter.messagebox.YES != tkinter.messagebox.askquestion(
                 parent=self.ui.get_toplevel(),
                 title=title,
-                message=psn.join(
-                    (
-                        "The edited ",
-                        "".join(
-                            (
-                                " score contains at least one illegal move ",
-                                "in PGN.\n\nPlease re-confirm request to ",
-                                "edit ",
-                            )
-                        ),
-                        ".",
-                    )
-                ),
+                message="".join(msg),
             ):
                 return
-            updater.value.set_game_source(self.pgn_score_source_name)
+            updater.value.gamesource = self.pgn_score_source
         original.set_database(editor.get_data_source().dbhome)
         updater.key.recno = original.key.recno
-        self.mark_partial_positions_to_be_recalculated(datasource=datasource)
+        self.mark_games_evaluated(datasource=datasource, allexcept=updater)
+        self.mark_all_cql_statements_not_evaluated(datasource=datasource)
         editor.edit()
         if self is self.ui_displayed_items.active_item:
             newkey = self.ui_displayed_items.adjust_edited_item(updater)
             if newkey:
                 self._set_properties_on_grids(newkey)
-        tags = original.value.collected_game.pgn_tags
-        tkinter.messagebox.showinfo(
-            parent=self.ui.get_toplevel(),
-            title=title,
-            message="".join(
-                (
-                    psn.title(),
-                    ' "',
-                    "  ".join([tags.get(k, "") for k in self.pgn_score_tags]),
-                    '" amended on database.',
+        if self.valid_cql_statements_exist(datasource=datasource):
+            self.run_cql_evaluator(datasource=datasource, ui=self.ui)
+            if self.ui.partial_items.count_items_in_stack():
+                active_item = self.ui.partial_items.active_item
+                active_item.refresh_game_list(
+                    key_recno=active_item.sourceobject.key.recno
                 )
-            ),
-        )
+        else:
+            tags = original.value.collected_game.pgn_tags
+            tkinter.messagebox.showinfo(
+                parent=self.ui.get_toplevel(),
+                title=title,
+                message="".join(
+                    (
+                        psn.title(),
+                        ' "',
+                        "  ".join(
+                            [tags.get(k, "") for k in self.pgn_score_tags]
+                        ),
+                        '" amended on database.',
+                    )
+                ),
+            )
