@@ -52,7 +52,7 @@ class Engine(EngineText):
         """Create widgets to display chess engine definition."""
         del boardfont
         panel = tkinter.Frame(
-            master=master, cnf=dict(borderwidth=2, relief=tkinter.RIDGE)
+            master=master, cnf={"borderwidth": 2, "relief": tkinter.RIDGE}
         )
         panel.grid_propagate(False)
         super().__init__(
