@@ -189,7 +189,12 @@ class EventSpec:
 
     # Game widget analyse events.
     # Inherited by Repertoire, GameEdit, and RepertoireEdit, widgets.
-    analyse_game = ("<Alt-KeyPress-a>", "Analyse", "Alt a")
+    analyse_game = ("<Control-KeyPress-g>", "Analyse Game", "Ctrl g")
+    analyse_game_position = (
+        "<Control-KeyPress-p>",
+        "Analyse Position",
+        "Ctrl p",
+    )
 
     # GameEdit widget editing events.
     # Inherited by RepertoireEdit widget.
@@ -609,7 +614,7 @@ class EventSpec:
         "",
     )  # No menu entry because character required.
 
-    # Partial widget partial position export events.
+    # Partial widget CQL query export events.
     # Inherited by PartialEdit widget.
     text_from_partial = (
         "<Control-Alt-KeyPress-Home>",
@@ -701,6 +706,13 @@ class EventSpec:
         4,
     )
     text_internal_format = ("<Alt-KeyPress-Home>", "Text", "Alt Home", 0)
+    menu_database_export_games_cql = ("", "PGN CQL scan", "", 4)
+    pgn_export_format_no_structured_comments = (
+        "<Control-KeyPress-KP_Home>",
+        "PGN no {[%]} comments",
+        "Ctrl KP_Home",
+        8,
+    )
 
     # ButtonPress event definitions.
     control_buttonpress_1 = ("<Control-ButtonPress-1>", "", "")
@@ -750,7 +762,8 @@ class EventSpec:
     menu_select_index_round = ("", "Round", "", 4)
 
     menu_game_new_game = ("", "New Game", "", 0)
-    menu_position_partial = ("", "Partial", "", 0)
+    menu_position_partial = ("", "Query", "", 0)
+    menu_position_show_query_engines = ("", "Show Query Engines", "", 11)
     menu_repertoire_opening = ("", "Opening", "", 0)
     menu_tools_board_style = ("", "Board Style", "", 6)
     menu_tools_board_fonts = ("", "Board Fonts", "", 6)
@@ -759,6 +772,12 @@ class EventSpec:
     menu_tools_show_game_analysis = ("", "Show Game Analysis", "", 5)
     menu_tools_hide_game_scrollbars = ("", "Hide Game Scrollbars", "", 1)
     menu_tools_show_game_scrollbars = ("", "Show Game Scrollbars", "", 2)
+    menu_tools_toggle_game_structured_comments = (
+        "",
+        "Toggle Game {[%...]} Comments",
+        "",
+        13,
+    )
     menu_tools_toggle_game_move_numbers = (
         "",
         "Toggle Game Move Numbers",
@@ -766,6 +785,7 @@ class EventSpec:
         12,
     )
     menu_tools_toggle__analysis_fen = ("", "Toggle Analysis Fen", "", 7)
+    menu_tools_toggle_entry_area_names = ("", "Toggle Entry Area Names", "", 7)
     menu_tools_toggle_single_view = ("", "Toggle Single View", "", 14)
     menu_engines_position_queues = ("", "Position Queues", "", 0)
     menu_engines_show_engines = ("", "Show Engines", "", 5)
