@@ -4,14 +4,20 @@
 
 """Customise edit toplevel to edit or insert chess game record."""
 
+import ast
+import tkinter
+
 from solentware_grid.gui.dataedit import DataEdit
 
 from pgn_read.core.parser import PGN
 from pgn_read.core.constants import TAG_WHITE, TAG_BLACK
 
+from ..core import constants
 from .gametoplevel import GameToplevel, GameToplevelEdit
 from .toplevelpgn import EditPGNToplevel
 from .constants import EMPTY_SEVEN_TAG_ROSTER
+from ..core import utilities
+from ..cql import runcql
 
 
 class GameDbEdit(EditPGNToplevel, DataEdit):
@@ -44,7 +50,7 @@ class GameDbEdit(EditPGNToplevel, DataEdit):
 
     pgn_score_name = "Game"
     pgn_score_tags = EMPTY_SEVEN_TAG_ROSTER
-    pgn_score_source = "Editor"
+    pgn_score_source = ""
 
     def __init__(
         self,
@@ -63,11 +69,29 @@ class GameDbEdit(EditPGNToplevel, DataEdit):
             oldobject=oldobject,
             newview=GameToplevelEdit(master=parent, ui=ui),
             title="",
-            oldview=GameToplevel(master=parent, ui=ui)
-            if showinitial
-            else showinitial,
+            oldview=(
+                GameToplevel(master=parent, ui=ui)
+                if showinitial
+                else showinitial
+            ),
         )
         self._initialize()
+        if utilities.is_game_import_in_progress_txn(
+            self.ui.database, self.newobject
+        ):
+            tkinter.messagebox.showinfo(
+                parent=parent,
+                title="Game import not complete",
+                message="".join(
+                    (
+                        "The selected game is displayed but attempts ",
+                        "to save edits to the game will be ",
+                        "rejected because some stages of importing this ",
+                        "game have not been completed.",
+                    )
+                ),
+            )
+            self.blockchange = True
 
     @property
     def ui_base_table(self):
@@ -89,7 +113,9 @@ class GameDbEdit(EditPGNToplevel, DataEdit):
         self._set_default_source_for_object(object_)
         view.set_position_analysis_data_source()
         view.collected_game = next(
-            PGN(game_class=view.gameclass).read_games(object_.get_srvalue())
+            PGN(game_class=view.gameclass).read_games(
+                ast.literal_eval(object_.get_srvalue()[0])
+            )
         )
         view.set_and_tag_item_text()
 
@@ -138,15 +164,74 @@ class GameDbEdit(EditPGNToplevel, DataEdit):
     # mark...recalculated starts and commits a transaction unconditionally.
     # No harm in using the same default as the 'super()' method.
     def put(self, commit=True):
-        """Mark partial position records for recalculation and return key."""
-        self.datasource.dbhome.mark_partial_positions_to_be_recalculated()
+        """Mark CQL query records for recalculation and return key.
+
+        If commit evaluates False caller is responsible for evaluating
+        CQL queries on the changes.
+
+        """
+        self.datasource.dbhome.mark_games_evaluated(
+            allexceptkey=(
+                self.newobject.key.recno
+                if self.newobject is not None
+                else None
+            )
+        )
+        self.datasource.dbhome.mark_all_cql_statements_not_evaluated()
         super().put(commit=commit)
+        if commit:
+            runcql.make_runcql(self.datasource.dbhome, self.ui, False)
+            if self.ui.partial_items.count_items_in_stack():
+                active_item = self.ui.partial_items.active_item
+                active_item.refresh_game_list(
+                    key_recno=active_item.sourceobject.key.recno
+                )
 
     # Resolve pylint message arguments-differ deferred.
     # Depends on detail of planned naming of methods as private if possible.
     # mark...recalculated starts and commits a transaction unconditionally.
     # No harm in using the same default as the 'super()' method.
     def edit(self, commit=True):
-        """Mark partial position records for recalculation and return key."""
-        self.datasource.dbhome.mark_partial_positions_to_be_recalculated()
+        """Mark CQL query records for recalculation and return key.
+
+        If commit evaluates False caller is responsible for evaluating
+        CQL queries on the changes.
+
+        """
+        self.datasource.dbhome.mark_games_evaluated(
+            allexceptkey=(
+                self.oldobject.key.recno
+                if self.oldobject is not None
+                else None
+            )
+        )
+        self.datasource.dbhome.mark_all_cql_statements_not_evaluated()
         super().edit(commit=commit)
+        if commit:
+            runcql.make_runcql(self.datasource.dbhome, self.ui, False)
+            if self.ui.partial_items.count_items_in_stack():
+                active_item = self.ui.partial_items.active_item
+                active_item.refresh_game_list(
+                    key_recno=active_item.sourceobject.key.recno
+                )
+
+    # This method forced by addition of second list element in Game record
+    # value, which breaks the 'class <Repertoire>(<Game>)' relationship in
+    # in classes in chessrecord module.
+    # Nowhere to put this in common with GameDisplayEdit.
+    def _construct_record_value(self, reference):
+        """Return record value for Game record."""
+        # Record value becomes {"file": "/", "game": ""} because the "file"
+        # value cannot be length zero when used as a key in a LMDB database.
+        # When "file" is a file name the "game" values will be 1, 2, 3,
+        # and so forth.
+        if reference[constants.GAME]:
+            game_number = ""
+        else:
+            game_number = reference[constants.GAME]
+        return repr(
+            [
+                repr(self.newview.get_score_error_escapes_removed()),
+                {constants.FILE: "/", constants.GAME: game_number},
+            ]
+        )
