@@ -2,22 +2,18 @@
 # Copyright 2013 Roger Marsh
 # Licence: See LICENCE (BSD licence)
 
-"""Customise delete toplevel to delete repertoire record.
-"""
+"""Customise delete toplevel to delete repertoire record."""
 
 from solentware_grid.gui.datadelete import DataDelete
 
-from solentware_misc.gui.exceptionhandler import ExceptionHandler
-
 from pgn_read.core.parser import PGN
 
 from ..core.constants import TAG_OPENING
 from .repertoiretoplevel import RepertoireToplevel
-from .toplevelpgn import DeletePGN
+from .toplevelpgn import DeletePGNToplevel
 
 
-class RepertoireDbDelete(ExceptionHandler, DeletePGN, DataDelete):
-
+class RepertoireDbDelete(DeletePGNToplevel, DataDelete):
     """Delete PGN text for repertoire from database.
 
     parent is used as the master argument in a RepertoireToplevel call.
@@ -30,7 +26,7 @@ class RepertoireDbDelete(ExceptionHandler, DeletePGN, DataDelete):
     Attribute pgn_score_name provides the name used in widget titles and
     message text.
 
-    Methods get_title_for_object and set_item, and properties ui_base_table;
+    Methods _get_title_for_object and _set_item, and properties ui_base_table;
     ui_items_in_toplevels; and ui, allow similar methods in various classes
     to be expressed identically and defined once.
 
@@ -38,35 +34,42 @@ class RepertoireDbDelete(ExceptionHandler, DeletePGN, DataDelete):
 
     pgn_score_name = "Repertoire"
 
-    def __init__(self, parent, oldobject, ui=None):
+    def __init__(self, parent=None, oldobject=None, ui=None):
         """Extend and create toplevel widget for deleting chess game."""
-        # Toplevel title set '' in __init__ and to proper value in initialize.
+        # Toplevel title set '' in __init__ and to proper value in _initialize.
         super().__init__(
-            oldobject, parent, RepertoireToplevel(master=parent, ui=ui), ""
+            instance=oldobject,
+            parent=parent,
+            oldview=RepertoireToplevel(master=parent, ui=ui),
+            title="",
         )
-        self.initialize()
+        self._initialize()
 
     @property
     def ui_base_table(self):
+        """Return the User Interface RepertoireGrid object."""
         return self.ui.base_repertoires
 
     @property
     def ui_items_in_toplevels(self):
+        """Return the User Interface objects in Toplevels."""
         return self.ui.games_and_repertoires_in_toplevels
 
     @property
     def ui(self):
+        """Return the User Interface object from 'read-only' view."""
         return self.oldview.ui
 
-    def set_item(self, view, object_):
-        self.set_default_source_for_object(object_)
+    def _set_item(self, view, object_):
+        """Populate view with the repertoire extracted from object_."""
+        self._set_default_source_for_object(object_)
         view.set_position_analysis_data_source()
         view.collected_game = next(
             PGN(game_class=view.gameclass).read_games(object_.get_srvalue())
         )
         view.set_and_tag_item_text()
 
-    def get_title_for_object(self, object_=None):
+    def _get_title_for_object(self, object_=None):
         """Return title for Toplevel containing a Repertoire object_.
 
         Default value of object_ is object attribute from DataDelete class.
@@ -78,7 +81,7 @@ class RepertoireDbDelete(ExceptionHandler, DeletePGN, DataDelete):
             return "  ".join(
                 (
                     self.pgn_score_name.join(("Delete ", ":")),
-                    object_.value.collected_game._tags[TAG_OPENING],
+                    object_.value.collected_game.pgn_tags[TAG_OPENING],
                 )
             )
         except TypeError:
@@ -90,10 +93,9 @@ class RepertoireDbDelete(ExceptionHandler, DeletePGN, DataDelete):
                 ("Delete ", " - name unknown or invalid")
             )
 
-    def set_default_source_for_object(self, object_=None):
+    def _set_default_source_for_object(self, object_=None):
         """Set default source for Toplevel containing a Repertoire object_.
 
         Default value of object_ is object attribute from DataDelete class.
 
         """
-        pass
