@@ -15,20 +15,19 @@ from .toplevelpgn import ToplevelPGN
 
 
 class RepertoireToplevel(ToplevelPGN, Repertoire):
-
-    """Customize Repertoire to be the single instance in a Toplevel widget."""
+    """Customize Repertoire seen in a ToplevelPGN widget."""
 
 
 class RepertoireToplevelEdit(ToplevelPGN, RepertoireEdit):
+    """Customize RepertoireEdit seen in a ToplevelPGN widget."""
 
-    """Customize RepertoireEdit to be the single instance in a Toplevel widget."""
-
-    def create_primary_activity_popup(self):
-        popup = super().create_primary_activity_popup()
-        self.add_pgn_navigation_to_submenu_of_popup(
+    def _create_primary_activity_popup(self):
+        """Create popup menu for a repertoire widget."""
+        popup = super()._create_primary_activity_popup()
+        self._add_pgn_navigation_to_submenu_of_popup(
             popup, index=self.analyse_popup_label
         )
-        self.add_pgn_insert_to_submenu_of_popup(
+        self._add_pgn_insert_to_submenu_of_popup(
             popup,
             include_ooo=True,
             include_move_rav=True,
