@@ -55,7 +55,6 @@ class _ToplevelText:
     def tidy_on_destroy(self):
         """Clear up after Toplevel destruction."""
         self.ui_items_in_toplevels.discard(self.oldview)
-        self.ui_base_table.selection.clear()
 
 
 class ShowText(_ToplevelText):
@@ -63,7 +62,7 @@ class ShowText(_ToplevelText):
 
     def dialog_ok(self):
         """Close the show record toplevel."""
-        if self.ui.database is None:
+        if self.ui.is_database_access_inhibited():
             self.destroy_dialog_on_ok_and_blockchange()
             return False
         return super().dialog_ok()
@@ -79,7 +78,7 @@ class DeleteText(_ToplevelText):
         started.
 
         """
-        if self.ui.database is None:
+        if self.ui.is_database_access_inhibited():
             self.status.configure(
                 text="Cannot delete because not connected to a database"
             )
@@ -117,7 +116,7 @@ class EditText(_ToplevelText):
         started.
 
         """
-        if self.ui.database is None:
+        if self.ui.is_database_access_inhibited():
             self.status.configure(
                 text="Cannot update because not connected to a database"
             )
@@ -129,4 +128,3 @@ class EditText(_ToplevelText):
         """Clear up after Toplevel destruction."""
         self.ui_items_in_toplevels.discard(self.oldview)
         self.ui_items_in_toplevels.discard(self.newview)
-        self.ui_base_table.selection.clear()
