@@ -2,12 +2,11 @@
 # Copyright 2015 Roger Marsh
 # Licence: See LICENCE (BSD licence)
 
-"""Import PGN file with apsw.chessapswdu to database."""
+"""Import PGN file with apsw.database_du to database."""
 
 
 if __name__ == "__main__":
-
     from .file_widget import FileWidget
-    from ..apsw.chessapswdu import ChessDatabase
+    from ..apsw.database_du import Database
 
-    FileWidget(ChessDatabase, "apsw")
+    FileWidget(Database, "apsw")
