@@ -2,12 +2,11 @@
 # Copyright 2021 Roger Marsh
 # Licence: See LICENCE (BSD licence)
 
-"""Import directory of PGN files with db.chessdbdu to database."""
+"""Import directory of PGN files with db.database_du to database."""
 
 
 if __name__ == "__main__":
-
     from .directory_widget import DirectoryWidget
-    from ..db.chessdbdu import chess_database_du
+    from ..db.database_du import database_du
 
-    DirectoryWidget(chess_database_du, "db")
+    DirectoryWidget(database_du, "db")
