@@ -2,12 +2,11 @@
 # Copyright 2023 Roger Marsh
 # Licence: See LICENCE (BSD licence)
 
-"""Import PGN file with db_tkinter.chessdbtkinterdu module."""
+"""Import PGN file with db_tkinter.database_du module."""
 
 
 if __name__ == "__main__":
-
     from .file_widget import FileWidget
-    from ..db_tkinter.chessdbtkinterdu import ChessDatabase
+    from ..db_tkinter.database_du import Database
 
-    FileWidget(ChessDatabase, "db_tcl")
+    FileWidget(Database, "db_tcl")
