@@ -2,12 +2,11 @@
 # Copyright 2021 Roger Marsh
 # Licence: See LICENCE (BSD licence)
 
-"""Import directory of PGN files with gnu.chessgnudu to database."""
+"""Import directory of PGN files with gnu.database_du to database."""
 
 
 if __name__ == "__main__":
-
     from .directory_widget import DirectoryWidget
-    from ..gnu.chessgnudu import chess_database_du
+    from ..gnu.database_du import database_du
 
-    DirectoryWidget(chess_database_du, "dbm.gnu")
+    DirectoryWidget(database_du, "dbm.gnu")
